/////////////////////////////////////////////////////
//// SampleApp02.c                               ////
//// Sample App for NxCore in C                  ////
//// Authors: Jeffrey Donovan, Jeremy Truelove   ////
//// Date: 02-08-2017                            ////
/////////////////////////////////////////////////////
//// Demonstrates:                               ////
//// 1) Handling the NxCore Symbol Spin Mesasge  ////
//// 2) Using NxOptionHdr                        ////
//// 3) Constructing Option Symbols in OSI or    ////
////    old OPRA format                          ////
/////////////////////////////////////////////////////

#include "StdAfx.h"
#include <stdio.h>          		// printf()
#include <NxCoreAPI_Wrapper_C.h>    // includes core NxCoreAPI.h

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreSymbolSpin(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);

int main(int argc, char* argv[])
{
	int returnValue;
	if(argc < 2) {
		printf("SampleApp02 dll-filename [nxTapeFilename]\n");
		exit(EXIT_SUCCESS);
	}

	printf("NxCore C SampleApp02 Start.\n");

	if(LoadNxCore(argv[1])) {
		// If a tape filename was passed in command line argument,
		// call ProcessTape with that argument.
		if(argv[2]){
			printf("Processing the tape: %s\n", argv[2]);
			returnValue = pfNxCoreProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		} else {
			returnValue = pfNxCoreProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		}

		// output return value from processTape(), helps in troubleshooting potential issues
		processReturnValue(returnValue);

		// Unload NxCore SO
		UnloadNxCore();
	}
	else {
	  printf("Failed to load NX Library. %s\n", dlerror());
      exit(EXIT_FAILURE);
	}

	printf("NxCore C SampleApp02 Stop.\n");
	exit(EXIT_SUCCESS);
}

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{

// Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
        // NxCore Status Message
        case NxMSG_STATUS:
             OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
                 break;

        // NxCore Symbol Spin Message
        case NxMSG_SYMBOLSPIN:
             OnNxCoreSymbolSpin(pNxCoreSys, pNxCoreMsg);
                 break;
          }


  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;
}

// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
// If a Minute has elapsed print the NxCore system time.
// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
// a real time value

	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
		(pNxCoreSys->nxTime.Hour < 24))
	{
	  printf("NxCore Time: %02d/%02d/%d  %02d:%02d:%02d\n",
			  pNxCoreSys->nxDate.Month,
			  pNxCoreSys->nxDate.Day,
			  pNxCoreSys->nxDate.Year,
			  pNxCoreSys->nxTime.Hour,
			  pNxCoreSys->nxTime.Minute,
			  pNxCoreSys->nxTime.Second);
	}

    // Print the specific NxCore status message
    switch( pNxCoreSys->Status ) 
    {
      case NxCORESTATUS_COMPLETE:
        printf("NxCore Complete Message.\n");
        break;

      case NxCORESTATUS_INITIALIZING:
		printf("NxCore Initialize Message.\n");
		printf("libnx.so version is v%u.%u.%u\n",
			NxCORE_VER_MAJOR(pNxCoreSys->DLLVersion),
			NxCORE_VER_MINOR(pNxCoreSys->DLLVersion),
			NxCORE_VER_BUILD(pNxCoreSys->DLLVersion));
        break;

      case NxCORESTATUS_SYNCHRONIZING:
        printf("NxCore Synchronizing Message.\n");
        break;

      case NxCORESTATUS_WAITFORCOREACCESS:
        printf("NxCore Wait For Access.\n");
        break;

      case NxCORESTATUS_RESTARTING_TAPE:
        printf("NxCore Restart Tape Message.\n");
        break;

      case NxCORESTATUS_ERROR:
        printf("NxCore Error.\n");
        break;

      case NxCORESTATUS_RUNNING:
        break;

    }
}
// OnNxCoreSymbolSpin: Function to handle NxCore Symbol Spin messages
//-------------------------------------------------------------------
void OnNxCoreSymbolSpin(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  // If this is an option, and there is a valid option header, use it
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&(pNxCoreMsg->coreHeader.pnxOptionHdr != NULL))
  {
        // Get the strike price from header info
        double StrikePrice;
        StrikePrice = pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice / 1000.0;

        char OptionSymbol[40];
        char UnderlyingStr[25];
        char SeriesStr[255];    
        UnderlyingStr[0]=SeriesStr[0]=OptionSymbol[0]=0;

        // Get the DateStrike string, Series string and Underlying symbol string
        // NOTE: VERY IMPORTANT TO NOT PASS NULL POINTERS IN AS ADDY FOR A STRING!!
        // SOME OF THESE STRINGS MAY OCCASIONALLY BE NULL 
        if (pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsUnderlying)
          strcpy(UnderlyingStr,pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsUnderlying->String);
        if (pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsSeriesChain)
          strcpy(SeriesStr,pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsSeriesChain->String);
        
        // If pnxsDateAndStrike->String[1] == ' ', then this symbol is in new OSI format.       
    if (pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[1]==' ')
    {
            // Construct OSI Symbol, take out 'o' from symbol root (String+1)   
                sprintf(OptionSymbol,"%-6s%02d%02d%02d%c%08d",
                        pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,
                            pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Year-2000,
                                pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Month,
                                pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Day,
                                (pNxCoreMsg->coreHeader.pnxOptionHdr->PutCall == 0) ? 'C' : 'P',
                                pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice);

                // Print the option symbol, underlying symbol and the series chain
                printf("Symbol Spin for: NxSym: %s  OSI Sym: %s  Strike: %0.2f  Underlying: %s  Series Chain: %s\n",
                           pNxCoreMsg->coreHeader.pnxStringSymbol->String,
                           OptionSymbol, StrikePrice, UnderlyingStr, SeriesStr);
    }

        // Otherwise the symbol is in old OPRA format.
        // Take out 'o' from symbol series (String+1)
        else
        {
          sprintf(OptionSymbol,"%s%c%c",
                     pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,
                     pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[0],
                 pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[1]);      

          // Print the option symbol, underlying symbol and the series chain
          printf("Symbol Spin for: NxSym: %s  OPRA Sym: %s  Strike: %0.2f  Underlying: %s  Series Chain: %s\n",
                        pNxCoreMsg->coreHeader.pnxStringSymbol->String,
                        OptionSymbol, StrikePrice, UnderlyingStr, SeriesStr);
        }
  }

  // Otherwise this is another equity type (stock, future, etc).
  else
  {
        // Print the symbol and listed exchange for this spin message
        printf("Symbol Spin for: %s  Listed Exg: %d\n",
                  pNxCoreMsg->coreHeader.pnxStringSymbol->String,
                  pNxCoreMsg->coreHeader.ListedExg);
 }
}

