/////////////////////////////////////////////////////
//// SampleApp05.c                               ////
//// Sample App for NxCore in C                  ////
//// Authors: Jeffrey Donovan, Jeremy Truelove   ////
//// Date: 02-08-2017                            ////
/////////////////////////////////////////////////////
//// Demonstrates:                               ////
//// Starting ProcessTape from a thread (pthread)////
//// Handling NxCore Trade and Quote Messages    ////
/////////////////////////////////////////////////////

#include "StdAfx.h"
#include <time.h>
#include <stdio.h>					// printf()
#include <NxCoreAPI_Wrapper_C.h>    // includes core NxCoreAPI.h

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreExgQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreMMQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);

char *filename;

#if defined(_WIN32) || defined(_WIN64)
typedef HANDLE pthread_t;
typedef void*(*NxThread)(char* dllFilename);

static int pthread_create(HANDLE *handle, void* notUsed, NxThread routine, void *arg) {
	*handle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)routine, arg, 0, NULL);
	return *handle ? 1 : 0;
}
#define pthread_exit(__arg) WaitForMultipleObjects(3, threads, TRUE, INFINITE);

#else
#include <unistd.h>
#include <pthread.h>
#endif

//our pthread void that will call our callback
void* TandQProc(void* dllFilename)
{
	int returnValue;

	if (LoadNxCore(dllFilename)) {

        // If a tape filename was passed in command line argument,
        // call ProcessTape with that argument. Use control flags
        // to eliminate Level1 quotes, Level2 quotes and all OPRA
        // quotes.
		if(filename){
			printf("Processing the tape: %s\n", filename);
			returnValue = pfNxCoreProcessTape(filename, NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		} else {
			returnValue = pfNxCoreProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		}

		// output return value from processTape(), helps in troubleshooting potential issues
		processReturnValue(returnValue);

	    // Unload NxCore SO
		UnloadNxCore();
	} else {
		printf("Failed to load NX Library. %s\n", dlerror());
	}
	return NULL;
}

int main(int argc, char* argv[])
{
  int thread01;

  if(argc < 2) {
	printf("SampleApp05 dll-filename [nxTapeFilename]\n");
	exit(EXIT_SUCCESS);
  }
  printf("NxCore C SampleApp05 Start.\n");

  filename = argv[2];

  //spawn a thread that will load nxcore, handle our TRADE and QUOTE messages
  //and then exit
  pthread_t threads[3];
  int i;
  for(i = 0; i < 3; i++){
	  thread01 = pthread_create(&threads[i], NULL, TandQProc, (void *)argv[1]);
  }
  

  if (thread01){
    printf("ERROR; return code from pthread_create() is %d\n", thread01);
    exit(EXIT_FAILURE);
  }

  pthread_exit(NULL);
  printf("NxCore C SampleApp05 Stop.\n");
  exit(EXIT_SUCCESS);
}

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
        // NxCore Status Message
        case NxMSG_STATUS:
             OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
                 break;

    // NxCore Trade Message
        case NxMSG_TRADE:
                 OnNxCoreTrade(pNxCoreSys, pNxCoreMsg);
                 break;

        // NxCore Level1 Quote Message
        case NxMSG_EXGQUOTE:
                 OnNxCoreExgQuote(pNxCoreSys, pNxCoreMsg);
                 break;

        // NxCore Level2 Quote Message
        case NxMSG_MMQUOTE:
                 OnNxCoreMMQuote(pNxCoreSys, pNxCoreMsg);
                 break;
  }
  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;
}

// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
	// a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) && (pNxCoreSys->nxTime.Hour < 24))
	{
		printf("NxCore Time: %02d/%02d/%d %02d:%02d:%02d\n",
				pNxCoreSys->nxDate.Month,
				pNxCoreSys->nxDate.Day,
				pNxCoreSys->nxDate.Year,
				pNxCoreSys->nxTime.Hour,
				pNxCoreSys->nxTime.Minute,
				pNxCoreSys->nxTime.Second);
	}
	// Print the specific NxCore status message
	switch (pNxCoreSys->Status) {
	case NxCORESTATUS_COMPLETE:
		printf("NxCore Complete Message.\n");
		break;
	case NxCORESTATUS_INITIALIZING:
		printf("NxCore Initialize Message.\n");
		printf("libnx.so version is v%u.%u.%u\n",
				NxCORE_VER_MAJOR(pNxCoreSys->DLLVersion),
				NxCORE_VER_MINOR(pNxCoreSys->DLLVersion),
				NxCORE_VER_BUILD(pNxCoreSys->DLLVersion));
		break;
	case NxCORESTATUS_SYNCHRONIZING:
		printf("NxCore Synchronizing Message.\n");
		break;
	case NxCORESTATUS_WAITFORCOREACCESS:
		printf("NxCore Wait For Access.\n");
		break;
	case NxCORESTATUS_RESTARTING_TAPE:
		printf("NxCore Restart Tape Message.\n");
		break;
	case NxCORESTATUS_ERROR:
		printf("NxCore Error.\n");
		break;
	case NxCORESTATUS_RUNNING:
		break;
    }
}

// OnNxCoreTrade: Function to handle NxCore Trade messages.     
//--------------------------------------------------------------
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  // Assign a pointer to the Trade data
  NxCoreTrade* Trade = ( NxCoreTrade*) &pNxCoreMsg->coreData.Trade;

  // Get the price and net change
  double Price = pfNxCorePriceToDouble(Trade->Price,Trade->PriceType);
  double NetChange = pfNxCorePriceToDouble(Trade->NetChange, Trade->PriceType);

  // Write out Symbol, Time, Price, NetChg, Size, Reporting Exg
  printf("Trade for Symbol: %s, Time: %02d:%02d:%02d Price: %0.2f NetChg: %0.2f Size: %u Exchg: %d\n",
	pNxCoreMsg->coreHeader.pnxStringSymbol->String,
	pNxCoreMsg->coreHeader.nxExgTimestamp.Hour,
	pNxCoreMsg->coreHeader.nxExgTimestamp.Minute,
	pNxCoreMsg->coreHeader.nxExgTimestamp.Second,
	Price,
	NetChange,
	Trade->Size,
	pNxCoreMsg->coreHeader.ReportingExg);
}

// OnNxCoreQuote: Function to handle NxCore ExgQuote messages.  
//--------------------------------------------------------------
void OnNxCoreExgQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  // Assign a pointer to the ExgQuote data
  NxCoreExgQuote* Quote = (NxCoreExgQuote*) &pNxCoreMsg->coreData.ExgQuote;

  // Get bid and ask price
  double Bid = pfNxCorePriceToDouble(Quote->coreQuote.BidPrice, Quote->coreQuote.PriceType);
  double Ask = pfNxCorePriceToDouble(Quote->coreQuote.AskPrice, Quote->coreQuote.PriceType);

  // Write out Symbol, Time, Bid, Ask, Bidsize, Asksize, Reporting Exg
  printf("ExgQuote for Symbol: %s, Time: %02d:%02d:%02d Bid: %0.2f Ask: %0.2f BidSize: %d AskSize: %d Exchg: %d\n",
                 pNxCoreMsg->coreHeader.pnxStringSymbol->String,
                 pNxCoreMsg->coreHeader.nxExgTimestamp.Hour, pNxCoreMsg->coreHeader.nxExgTimestamp.Minute, pNxCoreMsg->coreHeader.nxExgTimestamp.Second,
                 Bid, Ask, Quote->coreQuote.BidSize, Quote->coreQuote.AskSize,
                 pNxCoreMsg->coreHeader.ReportingExg);
}

// OnNxCoreMMQuote: Function to handle NxCore MMQuote messages. 
//--------------------------------------------------------------
void OnNxCoreMMQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  // Assign a pointer to the MMQuote data
  NxCoreMMQuote* Quote = (NxCoreMMQuote*) &pNxCoreMsg->coreData.MMQuote;
  // Get bid and ask price
  double Bid = pfNxCorePriceToDouble(Quote->coreQuote.BidPrice, Quote->coreQuote.PriceType);
  double Ask = pfNxCorePriceToDouble(Quote->coreQuote.AskPrice, Quote->coreQuote.PriceType);

  // Write out Symbol, MarketMaker, Time, Bid, Ask, Bidsize, Asksize, Reporting Exg
  printf("MMQuote for Symbol: %s, MarketMaker: %s Time: %02d:%02d:%02d Bid: %0.2f Ask: %0.2f BidSize: %d AskSize: %d Exchg: %d\n",
                 pNxCoreMsg->coreHeader.pnxStringSymbol->String,
                 Quote->pnxStringMarketMaker->String,
                 pNxCoreMsg->coreHeader.nxExgTimestamp.Hour,
				 pNxCoreMsg->coreHeader.nxExgTimestamp.Minute,
				 pNxCoreMsg->coreHeader.nxExgTimestamp.Second,
                 Bid,
				 Ask,
				 Quote->coreQuote.BidSize,
				 Quote->coreQuote.AskSize,
                 pNxCoreMsg->coreHeader.ReportingExg);
}


