/////////////////////////////////////////////////////
//// SampleApp06.c                               ////
//// Sample App for NxCore in C                  ////
//// Authors: Jeffrey Donovan, Jeremy Truelove   ////
//// Date: 02-08-2017                            ////
/////////////////////////////////////////////////////
//// Demonstrates:                               ////
//// Retriving StateTrades, StateExgQuotes,      ////
//// and StateMMQuotes                           ////
/////////////////////////////////////////////////////
//// NOTE: In this sample, we retrieve State     ////
////       items when NxCore delivers real time  ////
////       Trade, Quote and MMQuote messages.    ////
////       This is NOT the most effective way    ////
////       to get the streaming data (see        ////
////       Sample4). However it does effectively ////
////       demonstrate retrieving the State for  ////
////       any symbol.                           ////
/////////////////////////////////////////////////////

#include "StdAfx.h"
#include <stdio.h>					// printf()
#include <NxCoreAPI_Wrapper_C.h>    // includes core NxCoreAPI.h

// Couple forwards - functions are below
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnGetMMStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnGetExgQuoteStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnGetLastTradeStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);

// Main Entry Point for app.	
int main(int argc, char* argv[])
{
	int returnValue;

	if(argc < 2) {
		printf("SampleApp06 dll-filename [nxTapeFilename]\n");
		exit(EXIT_SUCCESS);
	}
    printf("NxCore C SampleApp6 Start.\n");

    // If we can load the NxCore library
	if (LoadNxCore(argv[1]))
    {
		// If a tape filename was passed in command line argument,
		// call ProcessTape with that argument.
		if (argv[2]) {
			printf("Processing the tape: %s\n", argv[2]);
			returnValue = pfNxCoreProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		} else {
			returnValue = pfNxCoreProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		}

		// output return value from processTape(), helps in troubleshooting potential issues
		processReturnValue(returnValue);

	    // Unload NxCore SO
		UnloadNxCore();
    }
	else {
	  printf("Failed to load NX Library. %s\n", dlerror());
      exit(EXIT_FAILURE);
	}

	printf("NxCore C SampleApp06 Stop.\n");
	exit(EXIT_SUCCESS);
}


// The NXCore Callback Function
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
	// NxCore Status Message
	case NxMSG_STATUS:
	     OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		 break;

    // NxCore Trade Message
	case NxMSG_TRADE:	  
		 OnGetLastTradeStates(pNxCoreSys, pNxCoreMsg);
		 break;

	// NxCore Level1 Quote Message
	case NxMSG_EXGQUOTE:	  
		 OnGetExgQuoteStates(pNxCoreSys, pNxCoreMsg);
		 break;

	// NxCore Level2 Quote Message
	case NxMSG_MMQUOTE:
		 OnGetMMStates(pNxCoreSys, pNxCoreMsg);
		 break;
  }
	  
  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;  
}


// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
    // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
	  printf("NxCore Time: %02d/%02d/%d  %02d:%02d:%02d\n",
		     pNxCoreSys->nxDate.Month,
			 pNxCoreSys->nxDate.Day,
			 pNxCoreSys->nxDate.Year,
             pNxCoreSys->nxTime.Hour,
			 pNxCoreSys->nxTime.Minute,
			 pNxCoreSys->nxTime.Second);
	}

	// Print the specific NxCore status message
	switch (pNxCoreSys->Status) {
	case NxCORESTATUS_COMPLETE:
		printf("NxCore Complete Message.\n");
		break;

	case NxCORESTATUS_INITIALIZING:
		printf("NxCore Initialize Message.\n");
		printf("libnx.so version is v%u.%u.%u\n",
			NxCORE_VER_MAJOR(pNxCoreSys->DLLVersion),
			NxCORE_VER_MINOR(pNxCoreSys->DLLVersion),
			NxCORE_VER_BUILD(pNxCoreSys->DLLVersion));
		break;

	case NxCORESTATUS_SYNCHRONIZING:
		printf("NxCore Synchronizing Message.\n");
		break;

	case NxCORESTATUS_WAITFORCOREACCESS:
		printf("NxCore Wait For Access.\n");
		break;

	case NxCORESTATUS_RESTARTING_TAPE:
		printf("NxCore Restart Tape Message.\n");
		break;

	case NxCORESTATUS_ERROR:
		printf("NxCore Error.\n");
		break;

	case NxCORESTATUS_RUNNING:
		break;

	}
}


// OnGetMMStates: Function to retrieve the current Market
// maker list for the issue, populated to the current time.
//----------------------------------------------------------
void OnGetMMStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  NxCoreStateMMQuotes pStatMMQuotes;

  // Retrieve MM Quote....make sure a valid return from function
  if (pfNxCoreStateGetMMQuotes(pNxCoreMsg->coreHeader.ListedExg,
			                &pStatMMQuotes,
					        pNxCoreMsg->coreHeader.pnxStringSymbol)==0)
  {

    printf("StateMMQuotes for %s, Count=%d\n",
 	       pNxCoreMsg->coreHeader.pnxStringSymbol->String, 
		   pStatMMQuotes.StateQuoteCount);

    // Cycle through all quotes in the state
    int loop;
    for (loop = 0; loop < pStatMMQuotes.StateQuoteCount; loop++)
    {  	  
      // Get bid and ask price
	  double Bid = pfNxCorePriceToDouble(pStatMMQuotes.StateMMQuotes[loop].BidPrice, pStatMMQuotes.PriceType);
	  double Ask = pfNxCorePriceToDouble(pStatMMQuotes.StateMMQuotes[loop].AskPrice, pStatMMQuotes.PriceType);
	  int BidSize = pStatMMQuotes.StateMMQuotes[loop].BidSize;
	  int AskSize = pStatMMQuotes.StateMMQuotes[loop].AskSize;

	  printf("Market Maker: %s Bid: %0.2f Ask: %0.2f AskSize: %d BidSize: %d\n", 
		     pStatMMQuotes.StateMMQuotes[loop].pnxStringMarketMaker->String,
		     Bid,
			 Ask,
			 BidSize,
			 AskSize);
	}
  }  
}


// OnGetExgQuoteStates: Function to retrieve the current Exg quote
// list for the issue, populated to the current time.
//----------------------------------------------------------
void OnGetExgQuoteStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  NxCoreStateExgQuotes pStatExgQuotes;
  NxString *RequestString=NULL;

  // If an option....
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&
	  (pNxCoreMsg->coreHeader.pnxOptionHdr))
	  RequestString=pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike;

  // otherwise a non-option
  else
	  RequestString=pNxCoreMsg->coreHeader.pnxStringSymbol;


  // Retrieve Exg Quote....make sure a valid return from function
  if (pfNxCoreStateGetExgQuotes(&pStatExgQuotes,RequestString)==0)
  {

    printf("StateExgQuotes for %s BestBidExg: %d BestAskExg: %d\n",
  	 	   pNxCoreMsg->coreHeader.pnxStringSymbol->String, 
		   pStatExgQuotes.BestBidExg,
		   pStatExgQuotes.BestAskExg);

    // Cycle through all quotes in the state
    int loop;
    for (loop = 0; loop < pStatExgQuotes.StateQuoteCount; loop++)
    {
      // Get bid and ask price
      double Bid = pfNxCorePriceToDouble(pStatExgQuotes.StateExgQuotes[loop].BidPrice, pStatExgQuotes.PriceType);
      double Ask = pfNxCorePriceToDouble(pStatExgQuotes.StateExgQuotes[loop].AskPrice, pStatExgQuotes.PriceType);
      int BidSize = pStatExgQuotes.StateExgQuotes[loop].BidSize;
      int AskSize = pStatExgQuotes.StateExgQuotes[loop].AskSize;
      unsigned short ReportingExg = pStatExgQuotes.StateExgQuotes[loop].ReportingExg;
      unsigned char Condition = pStatExgQuotes.StateExgQuotes[loop].QuoteCondition;
		  
      printf("Exg: %d Bid: %0.2f Ask: %0.2f BidSize: %d AskSize: %d Condition: %d\n",
		     ReportingExg,
			 Bid,
			 Ask,
			 BidSize,
			 AskSize,
			 Condition);
    }
  }
}


// OnGetLastTradeStates: Function to retrieve the current composite trade
// info for the issue, populated to the current time.
//----------------------------------------------------------
void OnGetLastTradeStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  NxCoreStateTrade pStatLastTrade;
  NxString *RequestString=NULL;

  // If an option....
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&
	  (pNxCoreMsg->coreHeader.pnxOptionHdr))
	  RequestString=pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike;

  // otherwise a non-option
  else
	  RequestString=pNxCoreMsg->coreHeader.pnxStringSymbol;

  // Retrieve Last Trade State....make sure a valid return from function
  if (pfNxCoreStateGetLastTrade(&pStatLastTrade,RequestString)==0)
  {
    printf("StateLastTrade for %s Open: %0.2f High: %0.2f Low: %0.2f Last: %0.2f Price: %0.2f NetChg: %0.2f TotalVol: %lld\n",
			pNxCoreMsg->coreHeader.pnxStringSymbol->String,
			pfNxCorePriceToDouble(pStatLastTrade.Open, pStatLastTrade.PriceType),
			pfNxCorePriceToDouble(pStatLastTrade.High, pStatLastTrade.PriceType),
			pfNxCorePriceToDouble(pStatLastTrade.Low, pStatLastTrade.PriceType),
			pfNxCorePriceToDouble(pStatLastTrade.Last, pStatLastTrade.PriceType),
			pfNxCorePriceToDouble(pStatLastTrade.Price, pStatLastTrade.PriceType),
			pfNxCorePriceToDouble(pStatLastTrade.NetChange, pStatLastTrade.PriceType),
		    pStatLastTrade.TotalVolume);	  
  }
}
