/////////////////////////////////////////////////////
//// SampleApp07.c                               ////
//// Sample App for NxCore in C                  ////
//// Authors: Jeffrey Donovan, Jeremy Truelove   ////
//// Date: 02-08-2017                            ////
/////////////////////////////////////////////////////
//// Demonstrates:                               ////
//// 1) User defined symbol spin function        ////
//// 2) Available NxCore Data file listings      ////
//// 3) Available NxCore DLL file listings       ////
//// 4) Retrieving strings from NxCore Tables    ////
//// 5) Get Date from N days                     ////
//// 6) Get N days from date                     ////
/////////////////////////////////////////////////////

#include "StdAfx.h"
#include <stdio.h>    				// printf()
#include <NxCoreAPI_Wrapper_C.h>    // includes core NxCoreAPI.h

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
//int OnUserListTapes(void *pUser, const NxCoreTapeFile *pTapeFile);
//int OnUserListAPIDLLs(void * pUser, const NxCoreAPIDLLFile *pDLLFile);
int OnUserSymbolSpin(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);

void GetDefinedString();
void GetTapeName();
void DateFromDays();
void DaysFromDate();

int main(int argc, char* argv[])
{
	int returnValue;

	if(argc < 2) {
		printf("SampleApp07 dll-filename [nxTapeFilename]\n");
		exit(EXIT_SUCCESS);
	}
	printf("NxCore C SampleApp07 Start.\n");

    // If we can load the NxCore library
	if (LoadNxCore(argv[1]))
    {
		// If a tape filename was passed in command line argument,
		// call ProcessTape with that argument.
		if (argv[2]) {
			printf("Processing the tape: %s\n", argv[2]);
			returnValue = pfNxCoreProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		} else {
			returnValue = pfNxCoreProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		}

		// output return value from processTape(), helps in troubleshooting potential issues
		processReturnValue(returnValue);

	    // Unload NxCore SO
		UnloadNxCore();
    }
	else {
	  printf("Failed to load NX Library. %s\n", dlerror());
      exit(EXIT_FAILURE);
	}

	printf("NxCore C SampleApp07 Stop.\n");
	exit(EXIT_SUCCESS);
}

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{

// Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
        // NxCore Status Message
        case NxMSG_STATUS:
             OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
                 break;
  }


  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;
}

// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
	// a real time value

	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE)
			&& (pNxCoreSys->nxTime.Hour < 24)) {
		printf("NxCore Time: %02d/%02d/%d  %02d:%02d:%02d\n",
				pNxCoreSys->nxDate.Month, pNxCoreSys->nxDate.Day,
				pNxCoreSys->nxDate.Year, pNxCoreSys->nxTime.Hour,
				pNxCoreSys->nxTime.Minute, pNxCoreSys->nxTime.Second);
	}

	// Print the specific NxCore status message
	switch (pNxCoreSys->Status) {
	// Capture a SYMBOL SPIN Message
	case NxCORESTATUS_SYMBOLSPIN:
		switch (pNxCoreSys->StatusData) {
		// If initial symbol spin is complete, test the functions in this
		// sample application
		case NxCSSYMBOLSPIN_COMPLETE:
			// User defined symbol spin
			printf("NxCore Initial Spin Complete Message.\n");
			printf("\nPress <Enter> to run a user define spin.\n");
			getchar();
			pfNxCoreSpinSymbols(NxSPIN_ALL_EXGS, 0, 1, (NxCoreCallback)OnUserSymbolSpin, 0);
			// Get defined string from NxCore tables
			printf("\nUserDefine Spin Complete!\n");
			printf("\nPress <Enter> to get defined strings for exchanges.\n");
			getchar();
			GetDefinedString();
			// Get current tape filename
			printf("\nPress <Enter> to get the Tape Name.\n");
			getchar();
			GetTapeName();
			// Get date from N days
			printf("\nPress <Enter> to get a date from N days.\n");
			getchar();
			DateFromDays();
			// Get N Days from date
			printf("\nPress <Enter> to get NDays from a date.\n");
			getchar();
			DaysFromDate();
			// Finish!
			printf("\nPress <Enter> to continue with tape.\n");
			getchar();
			break;
		}
		break;

	case NxCORESTATUS_COMPLETE:
		printf("NxCore Complete Message.\n");
		break;

	case NxCORESTATUS_INITIALIZING:
		printf("NxCore Initialize Message.\n");
		printf("libnx.so version is v%u.%u.%u\n",
				NxCORE_VER_MAJOR(pNxCoreSys->DLLVersion),
				NxCORE_VER_MINOR(pNxCoreSys->DLLVersion),
				NxCORE_VER_BUILD(pNxCoreSys->DLLVersion));
		break;

	case NxCORESTATUS_SYNCHRONIZING:
		DateFromDays();
		break;

	case NxCORESTATUS_WAITFORCOREACCESS:
		printf("NxCore Wait For Access.\n");
		break;

	case NxCORESTATUS_RESTARTING_TAPE:
		printf("NxCore Restart Tape Message.\n");
		break;

	case NxCORESTATUS_ERROR:
		printf("NxCore Error.\n");
		break;

	case NxCORESTATUS_RUNNING:
		break;

	}
}

// GetDefinedString: Get string description from NxCore table definitions.
//------------------------------------------------------------------------
void GetDefinedString()
{
  // Loop for 69 exchanges...
  int loop;
  for (loop = 1; loop < 70; loop++)
  {
        char Exg[260];

        // Get the defined name for this exchange
        strcpy(Exg,pfNxCoreGetDefinedString((int)NxST_EXCHANGE, loop));          

        // Print exchange num and name
        printf("Exg #%d = %s\n", loop,Exg);
  }
}

// GetTapeName: Get filename for current tape
//-------------------------------------------
void GetTapeName()
{
  char TapeName[260];
          
  // Get current tape name, convert to string type
  pfNxCoreGetTapeName(TapeName,254);
  
  // Print tape name
  if( strcmp("",TapeName) == 0 )
      printf("Realtime mode.\n");
  else
      printf("TapeName %s\n",TapeName);
}

// DateFromDays: Get date from N Days
//-----------------------------------
void DateFromDays()
{
  NxDate Date;
          
  // Set NDays to get date for
  Date.NDays=32000;

  // Call DateFromNDays function
  pfNxCoreDateFromNDays(&Date);

  // print NDays and date
  printf("Date for %d Days: %02d/%02d/%d\n",
	Date.NDays,
	Date.Month,
	Date.Day,
	Date.Year);
}
// DaysFromDate: Get N days from date
//-----------------------------------
void DaysFromDate()
{
  NxDate Date;

  // Set date to get NDays from
  Date.Month=1;
  Date.Day  =1;
  Date.Year = 2010;

  // Call DateFromYMD function
  pfNxCoreDateFromYMD(&Date);

  // Print date and NDays
  printf("Date %02d/%02d/%d = %d Days\n",
                 Date.Month,
                 Date.Day,
                 Date.Year,
                 Date.NDays);
}

// OnUserSymbolSpin:  User defined symbol spin callback
//------------------------------------------------------
int OnUserSymbolSpin(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{

  // Print the spin message sym, exg, time, date and session ID   
  printf("UserSymbol Spin for: %s  Listed Exg: %d  Time: %02d:%02d:%02d Date: %02d/%02d/%04d SessionID: %d\n",
                 pNxCoreMsg->coreHeader.pnxStringSymbol->String,
                 pNxCoreMsg->coreHeader.ListedExg,
                 pNxCoreMsg->coreHeader.nxExgTimestamp.Hour,
                 pNxCoreMsg->coreHeader.nxExgTimestamp.Minute,
                 pNxCoreMsg->coreHeader.nxExgTimestamp.Second,
                 pNxCoreMsg->coreHeader.nxSessionDate.Month,
                 pNxCoreMsg->coreHeader.nxSessionDate.Day,
                 pNxCoreMsg->coreHeader.nxSessionDate.Year,
                 pNxCoreMsg->coreHeader.SessionID);

  // Continue to next
  return NxCALLBACKRETURN_CONTINUE;
}

