//////////////////////////////////////////////////////
//// SampleApp08.c                                ////
//// Sample App for NxCore in C                   ////
//// Authors: Jeffrey Donovan, Jeremy Truelove    ////
//// Date: 02-08-2017                             ////
//////////////////////////////////////////////////////
//// Demonstrates:                                ////
//// 1) User defined symbol spin function         ////
//// 2) Setting UserData1 and UserData2 values    ////
//// 3) Retrieving UserData1 and UserData2 values ////
//// 4) Determining if a tape is OSI from the tape////
////    version number.                           ////
//// 5) Constructing Option Symbols in OSI or     ////
////    old OPRA format                           ////
//////////////////////////////////////////////////////

#include "StdAfx.h"
#include <stdio.h>    				// printf()
#include <NxCoreAPI_Wrapper_C.h>    // includes core NxCoreAPI.h

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreSymbolSpin(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
int OnUserSymbolSpin_NonOptions(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
int OnUserSymbolSpin_OptionsTraditional(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
int OnUserSymbolSpin_OptionsOSI(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);

// We'll use these to set the UserData1 and UserData1 members of NxString in the
// initial symbol spin, and retrieve the values from two UserDefined Spins
int MainCounter = 0;
int MainOptionCounter = 0;

unsigned char IsOSI=0;

int main(int argc, char* argv[])
{
	int returnValue;

	if(argc < 2) {
		printf("SampleApp08 dll-filename [nxTapeFilename]\n");
		exit(EXIT_SUCCESS);
	}
	printf("NxCore C SampleApp08 Start.\n");

    // If we can load the NxCore library
	if (LoadNxCore(argv[1]))
    {
		// If a tape filename was passed in command line argument,
		// call ProcessTape with that argument.
		if (argv[2]) {
			printf("Processing the tape: %s\n", argv[2]);
			returnValue = pfNxCoreProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		} else {
			returnValue = pfNxCoreProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		}

		// output return value from processTape(), helps in troubleshooting potential issues
		processReturnValue(returnValue);

	    // Unload NxCore SO
		UnloadNxCore();
    }
	else {
	  printf("Failed to load NX Library. %s\n", dlerror());
      exit(EXIT_FAILURE);
	}

	printf("NxCore C SampleApp08 Stop.\n");
	exit(EXIT_SUCCESS);
}

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{

// Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
	// NxCore Symbol Spin message
        case NxMSG_SYMBOLSPIN:
             OnNxCoreSymbolSpin(pNxCoreSys, pNxCoreMsg);
                 break;

        // NxCore Status Message
        case NxMSG_STATUS:
             OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
                 break;
  }


  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;
}

// ExtractTapeVersion  - Extract Tape Version Number
//----------------------------------------------------------
void ExtractTapeVersion(char *TapeString, char *VersionString, int *Major,int *Minor,int *Sub)
{
  if ((!TapeString)||(!VersionString)) return;

  char *pt=&TapeString[12]; // Version num starts at char 13 in TapeString
  char *pv=VersionString;
  char extracted[3];  
  char *pe=extracted;
  unsigned char vpos=0;
  
  // Extract version string and numeric components   
  while ((*pt)&&(*pt!='\n'))
  {
        // '.' seperates the major/minor/sub version nums
        if (*pt=='.')
        {
          // Extract Major and Minor versions
          *pe=0;          
          if (vpos==0) *Major=atoi(extracted);
          else         *Minor=atoi(extracted);  
                                                
          ++vpos;
          *pv++ = *pt++;          

          pe=extracted;
        }
        else    
          *pe++ = *pv++ = *pt++;
 } 
  // Set version string end to null
  *pv=0;

  // Extract final Sub version
  *pe=0;
  *Sub=atoi(extracted);
}

// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	char VString[12];
	int Major = 0, Minor = 0, Sub = 0;

	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
	// a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	(pNxCoreSys->nxTime.Hour < 24))
	{
		printf("NxCore Time: %02d/%02d/%d  %02d:%02d:%02d\n",
			pNxCoreSys->nxDate.Month,
			pNxCoreSys->nxDate.Day,
			pNxCoreSys->nxDate.Year,
			pNxCoreSys->nxTime.Hour,
			pNxCoreSys->nxTime.Minute,
			pNxCoreSys->nxTime.Second);
	}

	// Print the specific NxCore status message
	switch (pNxCoreSys->Status) {
	// Capture a SYMBOL SPIN Message
	case NxCORESTATUS_SYMBOLSPIN:
		switch (pNxCoreSys->StatusData) {

		// If initial symbol spin is complete, test the functions in this
		// sample application
		case NxCSSYMBOLSPIN_COMPLETE:

			// User defined symbol spin
			printf("NxCore Initial Spin Complete Message.\n");
			printf(
					"\nPress <Enter> to run a user define spin for non-options.\n");
			getchar();

			pfNxCoreSpinSymbols(NxSPIN_ALL_EXGS, 0, 1, (NxCoreCallback)OnUserSymbolSpin_NonOptions, 0);

			// User defined symbol spin
			printf("NxCore User Spin Complete Message.\n");
			printf("\nPress <Enter> to run a user define spin for options.\n");
			getchar();

			// If the tape is an OSI tape, point the spin to the OSI handler,
			// otherwise point the spin to the Traditional OPRA handler
			if (IsOSI)
				pfNxCoreSpinSymbols(NxSPIN_ALL_EXGS, 1, 1, (NxCoreCallback)OnUserSymbolSpin_OptionsOSI, 0);
			else
				pfNxCoreSpinSymbols(NxSPIN_ALL_EXGS, 1, 1, (NxCoreCallback)OnUserSymbolSpin_OptionsTraditional, 0);

			// Finish!
			printf("\nPress <Enter> to continue with tape.\n");
			getchar();
			break;
		}
		break;
	case NxCORESTATUS_COMPLETE:
		printf("NxCore Complete Message.\n");
		break;

	case NxCORESTATUS_INITIALIZING:
		printf("NxCore Initialize Message.\n");
		printf("libnx.so version is v%u.%u.%u\n",
				NxCORE_VER_MAJOR(pNxCoreSys->DLLVersion),
				NxCORE_VER_MINOR(pNxCoreSys->DLLVersion),
				NxCORE_VER_BUILD(pNxCoreSys->DLLVersion));
		break;

	case NxCORESTATUS_SYNCHRONIZING:
		printf("NxCore Synchronizing Message.\n");
		break;

	case NxCORESTATUS_WAITFORCOREACCESS:
		printf("NxCore Wait For Access.\n");
		break;

	case NxCORESTATUS_RESTARTING_TAPE:
		printf("NxCore Restart Tape Message.\n");
		break;

	case NxCORESTATUS_ERROR:
		printf("NxCore Error.\n");
		break;

	case NxCORESTATUS_RUNNING:
		break;
		// Tape Open Message. Get the tape version number
	case NxCORESTATUS_TAPEOPEN:
		// Extract tape version from TapeDisplay string
		ExtractTapeVersion((char *) pNxCoreSys->TapeDisplay, VString, &Major,
				&Minor, &Sub);

		printf("NxCore Tape Open Message.\n");
		printf("\nTape Version String: %s  Tape Version Numeric: %d %d %d  System Date %02d/%02d/%d\n",
				VString,
				Major,
				Minor,
				Sub,
				pNxCoreSys->nxDate.Month,
				pNxCoreSys->nxDate.Day,
				pNxCoreSys->nxDate.Year);

		// If tape version is greater than 1.20.x then this is an OSI tape
		if ((Major > 1) || ((Major == 1) && (Minor > 20)))
			IsOSI = 1;

		printf("\nPress <Enter> to Continue...\n");
		getchar();
		break;
	}
}

// OnNxCoreSymbolSpin: Function to handle NxCore Symbol Spin messages
//-------------------------------------------------------------------
void OnNxCoreSymbolSpin(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  int UData1 = 0;
  int UData2 = 0;

  // If this is an option, and there is a valid option header, use it
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&(pNxCoreMsg->coreHeader.pnxOptionHdr != NULL))
  {             
        char OptionSymbol[40];

        // SET UserData1 for the option to MainOptionCounter (increment the counter as we do).
        UData1=pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData1 = ++MainOptionCounter;

        // SET UserData2 to some odd value based on Nx Clock
        UData2=pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData2 = (pNxCoreSys->nxTime.Second * pNxCoreSys->nxTime.Millisecond);


        // If tape is determined to be an OSI tape...
    if (IsOSI)          
    {
		// Construct OSI symbol, take out 'o' from symbol root (String+1)
		sprintf(OptionSymbol,"%-6s%02d%02d%02d%c%08d",
			pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,
			pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Year-2000,
			pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Month,
			pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Day,
			(pNxCoreMsg->coreHeader.pnxOptionHdr->PutCall == 0) ? 'C' : 'P',
			pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice);

		// Print the option symbol and user data
		printf("Initial Symbol Spin for: NxSym: %s OSI Sym: %s UserData1: %d UserData2: %d\n",
			pNxCoreMsg->coreHeader.pnxStringSymbol->String,
			OptionSymbol,
			UData1,
			UData2);
    }
        // Otherwise the symbol is in old OPRA format.
        // Take out 'o' from symbol series (String+1)
        else
        {
        	sprintf(OptionSymbol,"%s%c%c",
			pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,
			pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[0],
			pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[1]);

			// Print the option symbol and user data
			printf("Initial Symbol Spin for: NxSym: %s OPRA Sym: %s UserData1: %d UserData2: %d\n",
				pNxCoreMsg->coreHeader.pnxStringSymbol->String,
				OptionSymbol,
				UData1,
				UData2);
        }
  }

  // Otherwise this is another equity type (stock, future, etc).
  else 
  {
        // SET UserData1 to the MainCounter (increment the counter as we do).
        UData1=pNxCoreMsg->coreHeader.pnxStringSymbol->UserData1 = ++MainCounter;

        // SET UserData2 to some odd value based on Nx Clock
        UData2 = pNxCoreMsg->coreHeader.pnxStringSymbol->UserData2 = (pNxCoreSys->nxTime.Second * pNxCoreSys->nxTime.Millisecond);

        // Print the symbol and listed exchange for this spin message
        printf("Initial Symbol Spin for: %s Listed Exg: %d UserData1=%d UserData2=%d\n",
                  pNxCoreMsg->coreHeader.pnxStringSymbol->String,
                  pNxCoreMsg->coreHeader.ListedExg,
				  UData1,
				  UData2);
  }
}

// OnUserSymbolSpin:  User defined symbol spin callback for non options
//---------------------------------------------------------------------
int OnUserSymbolSpin_NonOptions(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  int UData1 = 0;
  int UData2 = 0;

  // Retrieve the UseerData1 and UserData2 values set in the initial spin
  UData1 = pNxCoreMsg->coreHeader.pnxStringSymbol->UserData1;
  UData2 = pNxCoreMsg->coreHeader.pnxStringSymbol->UserData2;

  // Print the symbol and listed exchange for this spin message
  printf("User Symbol Spin for: %s Listed Exg: %d UserData1=%d UserData2=%d\n",
          pNxCoreMsg->coreHeader.pnxStringSymbol->String,
          pNxCoreMsg->coreHeader.ListedExg,
		  UData1,
		  UData2);
          
  // Continue to next
  return NxCALLBACKRETURN_CONTINUE;
}

// OnUserSymbolSpin:  User defined symbol spin callback for tape with traditional OPRA option symbols
//------------------------------------------------------
int OnUserSymbolSpin_OptionsTraditional(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  int UData1 = 0;
  int UData2 = 0;

  
  // If this is an option, and there is a valid option header, use it
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&(pNxCoreMsg->coreHeader.pnxOptionHdr != NULL))
  {    
	char OptionSymbol[25];

	// Retrieve the UseerData1 and UserData2 values set in the initial spin
	UData1 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData1;
	UData2 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData2;

	sprintf(OptionSymbol,"%s%c%c",
	pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,
	pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[0],
	pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[1]);

	// Print the option symbol and user data
	printf("UserSymbol Spin for: NxSym: %s OPRA Sym: %s UserData1: %d UserData2: %d\n",
		pNxCoreMsg->coreHeader.pnxStringSymbol->String,
		OptionSymbol,
		UData1,
		UData2);
  }
          
  // Continue to next
  return NxCALLBACKRETURN_CONTINUE;
}

// OnUserSymbolSpin:  User defined symbol spin callback for tape with OSI option symbols
//------------------------------------------------------
int OnUserSymbolSpin_OptionsOSI(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  int UData1 = 0;
  int UData2 = 0;

  // If this is an option, and there is a valid option header, use it
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&(pNxCoreMsg->coreHeader.pnxOptionHdr != NULL))
  {    
		char OptionSymbol[40];

		// Retrieve the UseerData1 and UserData2 values set in the initial spin
		UData1 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData1;
		UData2 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData2;

		// Construct OSI Symbol, take out 'o' from symbol root (String+1)
		sprintf(OptionSymbol,"%-6s%02d%02d%02d%c%08d",
			pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,
			pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Year-2000,
			pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Month,
			pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Day,
			(pNxCoreMsg->coreHeader.pnxOptionHdr->PutCall == 0) ? 'C' : 'P',
			pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice);

		// Print the option symbol and user data
		printf("UserSymbol Spin for: NxSym: %s OSI Sym: %s UserData1: %d UserData2: %d\n",
			pNxCoreMsg->coreHeader.pnxStringSymbol->String,
			OptionSymbol,
			UData1,
			UData2);
  }

  // Continue to next
  return NxCALLBACKRETURN_CONTINUE;
}

