//////////////////////////////////////////////////////
//// SampleApp10.c                                ////
//// Sample App for NxCore in C                   ////
//// Authors: Jeffrey Donovan, Jeremy Truelove    ////
//// Date: 02-08-2017                             ////
//////////////////////////////////////////////////////
//// Demonstrates:                                ////
//// 1) Saving to a NxCore State Tape             ////
//// 2) Running from an NxCore State Tape         ////
//////////////////////////////////////////////////////

#include "StdAfx.h"
#include <stdio.h>    				// printf()
#include <NxCoreAPI_Wrapper_C.h>    // includes core NxCoreAPI.h

// Couple forwards - functions are below
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);

unsigned char SavedState=0;
char StateFileFilename[255];

// Main Entry Point for app.
int main(int argc, char* argv[])
{
	int returnValue;

	if(argc < 2) {
		printf("SampleApp10 dll-filename [nxTapeFilename]\n");
		exit(EXIT_SUCCESS);
	}
    printf("NxCore C SampleApp10 Start.\n");

	// If we can load the NxCore library
	if (LoadNxCore(argv[1]))
	{
		strcpy(StateFileFilename, "savedstate.nxs");

		printf("\nThe application will process trades until 7:00 AM.\n");
		printf("It will then save the state, exit and reload from the saved state file.\n\n");
		printf("Press <ENTER> to continue.\n");
		getchar();

		// If a tape filename was passed in command line argument,
		// call ProcessTape with that argument.
		if (argv[2]) {
			printf("Processing the tape: %s\n", argv[2]);
			returnValue = pfNxCoreProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		} else {
			returnValue = pfNxCoreProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		}

		// output return value from processTape(), helps in troubleshooting potential issues
		processReturnValue(returnValue);

		// The current process has exited and the state tape has been saved!
		printf("Tape state saved and process exited.\n");
		printf("Press <ENTER> to Load the state and continue.\n");
		getchar();

		// Reset this flag so we dont stop in the next call to ProcessTape
		SavedState=0;

		// Process the tape from a state file
		returnValue = pfNxCoreProcessTape(StateFileFilename, NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);

		// output return value from processTape() to the state file, helps in troubleshooting potential issues
		processReturnValue(returnValue);

		// Unload NxCore SO
		UnloadNxCore();
    }
	else {
	  printf("Failed to load NX Library. %s\n", dlerror());
      exit(EXIT_FAILURE);
	}

	printf("NxCore C SampleApp01 Stop.\n");
	exit(EXIT_SUCCESS);
}


// The NXCore Callback Function	
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{ 

  // If this flag is set, NxCore has saved the state. Note that from the time
  // the NxCORESTATUS_SAVING_STATE message was received, the function issued
  // one moreNxCALLBACKRETURN_CONTINUE, and the next time in we will exit, thus
  // allowing NxCore to properly save the state and to exit automatically.
  if (SavedState)
      return NxCALLBACKRETURN_STOP;

  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
		// NxCore Status Message
	case NxMSG_STATUS:
	     OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		 break;

    // NxCore Trade Message
	case NxMSG_TRADE:
		 OnNxCoreTrade(pNxCoreSys, pNxCoreMsg);
		 break;
  }

  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;
}


// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{

	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
    // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
	  printf("NxCore Time: %02d/%02d/%d  %02d:%02d:%02d\n",
		     pNxCoreSys->nxDate.Month,
			 pNxCoreSys->nxDate.Day,
			 pNxCoreSys->nxDate.Year,
             pNxCoreSys->nxTime.Hour,
			 pNxCoreSys->nxTime.Minute,
			 pNxCoreSys->nxTime.Second);

	  // LOOK! If it's 7:00:00 AM SAVE THE CURRENT STATE TO A STATE FILE!
	  if (pNxCoreSys->nxTime.Hour==7 && pNxCoreSys->nxTime.Minute==0 && pNxCoreSys->nxTime.Second==0)
	  {
		// NOTE: This application will only work properly with the NxSAVESTATE_ONEPASS flag setting,
		// as we exit the tape exactly one pass after the file is to be saved. Should you modify this
		// flag the application will need to also be modified accordingly.
		pfNxCoreSaveState(StateFileFilename,NxSAVESTATE_ONEPASS);
	  }
	}
	
	// Print the specific NxCore status message
    switch( pNxCoreSys->Status ) 
    {
	  // NxCore will save the state this pass. Tell user and wait for keypress
  	  case NxCORESTATUS_SAVING_STATE:
		   printf("\nNxCore Will Save the State Now!\n");
		   printf("Press <ENTER> to Continue.\n");
		   getchar();

		   // Set the state saved flag
		   SavedState=1;
		   break;

	  // NxCore Has been loaded from a state tape. Tell user and wait for keypress
	  case NxCORESTATUS_LOADED_STATE:
		   printf("NxCore Has Been Loaded From A State!\n");
		   printf("Press <ENTER> to continue processing trades.\n");
		   getchar();
		   break;

	  case NxCORESTATUS_COMPLETE:
		   printf("NxCore Complete Message.\n");		   
		   break;

      case NxCORESTATUS_INITIALIZING:
		   printf("libnx.so version is v%u.%u.%u\n",
			   NxCORE_VER_MAJOR(pNxCoreSys->DLLVersion),
			   NxCORE_VER_MINOR(pNxCoreSys->DLLVersion),
			   NxCORE_VER_BUILD(pNxCoreSys->DLLVersion));
		   printf("NxCore Initialize Message.\n");		   
		   break;

	  case NxCORESTATUS_SYNCHRONIZING:
		   printf("NxCore Synchronizing Message.\n");
		   break;

	  case NxCORESTATUS_WAITFORCOREACCESS:
		   printf("NxCore Wait For Access.\n");
		   break;

	  case NxCORESTATUS_RESTARTING_TAPE:
		   printf("NxCore Restart Tape Message.\n");
		   break;

      case NxCORESTATUS_ERROR:
		   printf("NxCore Error.\n");
		   break;
 	  
      case NxCORESTATUS_RUNNING:		   
		   break;

    }
}

// OnNxCoreTrade: Function to handle NxCore Trade messages.	
//--------------------------------------------------------------
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  if (pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='e')
  {
    // Assign a pointer to the Trade data
    NxCoreTrade* Trade = ( NxCoreTrade*) &pNxCoreMsg->coreData.Trade;

    // Get the price and net change
    double Price = pfNxCorePriceToDouble(Trade->Price,Trade->PriceType);
    double NetChange = pfNxCorePriceToDouble(Trade->NetChange, Trade->PriceType);

    // Write out Symbol, Time, Price, NetChg, Size, Reporting Exg
    printf("Trade for Symbol: %s, Time: %02d:%02d:%02d.%03d  Price: %0.2f  NetChg: %0.2f  Size: %d  Exchg: %d\n",
		   pNxCoreMsg->coreHeader.pnxStringSymbol->String,
		   pNxCoreMsg->coreHeader.nxExgTimestamp.Hour,
		   pNxCoreMsg->coreHeader.nxExgTimestamp.Minute,
		   pNxCoreMsg->coreHeader.nxExgTimestamp.Second,
		   pNxCoreMsg->coreHeader.nxExgTimestamp.Millisecond,
		   Price,
		   NetChange,
		   Trade->Size,
		   pNxCoreMsg->coreHeader.ReportingExg);
  }
}
