/////////////////////////////////////////////////////
//// SampleApp11.c                               ////
//// Sample App for NxCore in C                  ////
//// Authors: Jeffrey Donovan, Jeremy Truelove   ////
//// Date: 02-08-2017                            ////
/////////////////////////////////////////////////////
//// Demonstrates:                               ////
//// Handling NxCore Trade and Quote Messages    ////
//// Including NxCore3 extended fields and micros////
/////////////////////////////////////////////////////

#include "StdAfx.h"
#include <stdio.h>    				// printf()
#include <NxCoreAPI_Wrapper_C.h>    // includes core NxCoreAPI.h

// JTime Time Structure
typedef struct
{
  unsigned char Hour;
  unsigned char Minute;
  unsigned char Second;
  unsigned short Millisecond;
  unsigned short Microsecond;
  unsigned int MsOfDay;
  __int64 MicrosOfDay;
} JTime;

// Get Micro Diff from MsOfDay
#define JTime_MicroDiffFromMSofDay(micros,ms) micros-((__int64)ms*1000)

// Couple forwards - functions are below
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);

// Main Entry Point for app.
int main(int argc, char* argv[])
{
	int returnValue;

	if(argc < 2) {
		printf("SampleApp11 dll-filename [nxTapeFilename]\n");
		exit(EXIT_SUCCESS);
	}
   printf("NxCore C SampleApp11 Start.\n");

    // If we can load the NxCore library
	if (LoadNxCore(argv[1]))
    {
		// If a tape filename was passed in command line argument,
		// call ProcessTape with that argument.
		if (argv[2]) {
			printf("Processing the tape: %s\n", argv[2]);
			returnValue = pfNxCoreProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		} else {
			returnValue = pfNxCoreProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		}

		// output return value from processTape(), helps in troubleshooting potential issues
		processReturnValue(returnValue);

	    // Unload NxCore SO
		UnloadNxCore();
    }
	else {
	  printf("Failed to load NX Library. %s\n", dlerror());
      exit(EXIT_FAILURE);
	}

	printf("NxCore C SampleApp11 Stop.\n");
	exit(EXIT_SUCCESS);
}


// The NXCore Callback Function
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
	// NxCore Status Message
	case NxMSG_STATUS:
	     OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		 break;

	// NxCore Trade Message
	case NxMSG_TRADE:
		 OnNxCoreTrade(pNxCoreSys, pNxCoreMsg);
		 break;

  }

  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;
}


// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
        // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
	  printf("NxCore Time: %02d/%02d/%d  %02d:%02d:%02d\n",
		     pNxCoreSys->nxDate.Month,
			 pNxCoreSys->nxDate.Day,
			 pNxCoreSys->nxDate.Year,
             pNxCoreSys->nxTime.Hour,
			 pNxCoreSys->nxTime.Minute,
			 pNxCoreSys->nxTime.Second);
	}

	// Print the specific NxCore status message
	switch (pNxCoreSys->Status) {
	case NxCORESTATUS_COMPLETE:
		printf("NxCore Complete Message.\n");
		break;

		//print the API version on initialization
	case NxCORESTATUS_INITIALIZING:
		printf("NxCore Initialize Message.\n");
		printf("libnx.so version is v%u.%u.%u\n",
				NxCORE_VER_MAJOR(pNxCoreSys->DLLVersion),
				NxCORE_VER_MINOR(pNxCoreSys->DLLVersion),
				NxCORE_VER_BUILD(pNxCoreSys->DLLVersion));
		break;

	case NxCORESTATUS_SYNCHRONIZING:
		printf("NxCore Synchronizing Message.\n");
		break;

	case NxCORESTATUS_WAITFORCOREACCESS:
		printf("NxCore Wait For Access.\n");
		break;

	case NxCORESTATUS_RESTARTING_TAPE:
		printf("NxCore Restart Tape Message.\n");
		break;

	case NxCORESTATUS_ERROR:
		printf("NxCore Error.\n");
		break;

	case NxCORESTATUS_RUNNING:
		break;

	}
}

// Convert MicrosecondOfDay to JTime
void JTime_MicrosOfDayToTime(unsigned __int64 MicrosOfDay,JTime *Time)
{
   Time->MicrosOfDay=MicrosOfDay;

   Time->MsOfDay=(unsigned int)(MicrosOfDay/1000);
   unsigned int Seconds=Time->MsOfDay/1000;

   Time->Hour       =(unsigned char) (Seconds/(60*60));
   Time->Minute     =(unsigned char) (Seconds/60%60);
   Time->Second     =(unsigned char) (Seconds%60);
   Time->Millisecond=(unsigned short)(Time->MsOfDay % 1000);
   Time->Microsecond=(unsigned short)(JTime_MicroDiffFromMSofDay(MicrosOfDay,Time->MsOfDay));
}

// OnNxCoreTrade: Function to handle NxCore Trade messages.
//--------------------------------------------------------------
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  if (pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='e')
  {
    // Assign a pointer to the Trade data
    NxCoreTrade* Trade = ( NxCoreTrade*) &pNxCoreMsg->coreData.Trade;

    // Get the price and net change
    double Price = pfNxCorePriceToDouble(Trade->Price,Trade->PriceType);
    double NetChange = pfNxCorePriceToDouble(Trade->NetChange, Trade->PriceType);

    // Write out Symbol, Time, Price, NetChg, Size, Reporting Exg
    if (Trade->ExtTradeConditions[0]!=0)
      printf("Trade for Symbol: %s, Time: %02d:%02d:%02d.%03d  Price: %0.2f  NetChg: %0.2f  Size: %d  Exchg: %d  TC: (%d) ( %d,%d,%d,%d )\n",
		 pNxCoreMsg->coreHeader.pnxStringSymbol->String,
		 pNxCoreMsg->coreHeader.nxExgTimestamp.Hour,
		 pNxCoreMsg->coreHeader.nxExgTimestamp.Minute,
		 pNxCoreMsg->coreHeader.nxExgTimestamp.Second,
		 pNxCoreMsg->coreHeader.nxExgTimestamp.Millisecond,
		 Price,
		 NetChange,
		 Trade->Size,
		 pNxCoreMsg->coreHeader.ReportingExg,
         Trade->TradeCondition,
         Trade->ExtTradeConditions[0],
		 Trade->ExtTradeConditions[1],
		 Trade->ExtTradeConditions[2],
		 Trade->ExtTradeConditions[3]);
    else
      printf("Trade for Symbol: %s, Time: %02d:%02d:%02d.%03d  Price: %0.2f  NetChg: %0.2f  Size: %d  Exchg: %d  TC: (%d)\n",
		 pNxCoreMsg->coreHeader.pnxStringSymbol->String,
		 pNxCoreMsg->coreHeader.nxExgTimestamp.Hour,
		 pNxCoreMsg->coreHeader.nxExgTimestamp.Minute,
		 pNxCoreMsg->coreHeader.nxExgTimestamp.Second,
		 pNxCoreMsg->coreHeader.nxExgTimestamp.Millisecond,
		 Price,
		 NetChange,
		 Trade->Size,
		 pNxCoreMsg->coreHeader.ReportingExg,
         Trade->TradeCondition);

  if (pNxCoreMsg->pnxCore3Ext)
    {
      JTime SRCTime,PTP1Time,PTP2Time;
      JTime_MicrosOfDayToTime(pNxCoreMsg->pnxCore3Ext->MicrosOfDaySRC, &SRCTime);
      JTime_MicrosOfDayToTime(pNxCoreMsg->pnxCore3Ext->MicrosOfDayPTP1,&PTP1Time);
      JTime_MicrosOfDayToTime(pNxCoreMsg->pnxCore3Ext->MicrosOfDayPTP2,&PTP2Time);

      printf("[SRC TIME: %02d:%02d:%02d.%03d.%03d]  [PTP1 TIME: %02d:%02d:%02d.%03d.%03d]  [PTP2 TIME: %02d:%02d:%02d.%03d.%03d]\n",
        SRCTime.Hour,
		SRCTime.Minute,
		SRCTime.Second,
		SRCTime.Millisecond,
		SRCTime.Microsecond,

        PTP1Time.Hour,
		PTP1Time.Minute,
		PTP1Time.Second,
		PTP1Time.Millisecond,
		PTP1Time.Microsecond,

        PTP2Time.Hour,
		PTP2Time.Minute,
		PTP2Time.Second,
		PTP2Time.Millisecond,
		PTP2Time.Microsecond);
    }
  }
}
