////////////////////////////////////////////////////
//// SampleApp2.cs                              ////
//// Sample App for C# NxCore API Wrapper       ////
//// Author: Jeffrey Donovan                    ////
//// Date: 12-22-09                             ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// 1) Handling the NxCore Symbol Spin Message ////
//// 2) Using NxOptionHdr                       ////
//// 3) Constructing Option Symbols in OSI or   ////
////    old OPRA format                         ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;

// Use the NxCoreAPI namespace
using NxCoreAPI;


namespace SampleApp2
{ 

  class Program
  {

	// Main Entry Point for app.	
	//--------------------------
	static void Main(string[] args)
	{
	  Console.WriteLine("NxCore CSharp SampleApp2 Start.");

	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument.
	  if (args.Length > 0)
		NxCore.ProcessTape(args[0],
						   null, (uint)0, 0,
						   Program.OnNxCoreCallback);

	  // Otherwise start NxCore with the current day's tape (IE Realtime)
	  else
		NxCore.ProcessTape("",
						   null, (uint)0, 0,
						   Program.OnNxCoreCallback);
	  
	  Console.WriteLine("NxCore CSharp SampleApp2 Stop.");		
	}
	

	// The NXCore Callback Function	
	//-----------------------------
	static unsafe int OnNxCoreCallback(IntPtr pSys, IntPtr pMsg)
	{
	  // Alias structure pointers to the pointers passed in.
	  NxCoreSystem* pNxCoreSys = (NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg = (NxCoreMessage*)pMsg;

	  // Do something based on the message type
	  switch (pNxCoreMsg->MessageType)
	  {
		// NxCore Status Message
		case NxCore.NxMSG_STATUS:
		  OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		  break;

		// NxCore Symbol Spin Message
		case NxCore.NxMSG_SYMBOLSPIN:
		  OnNxCoreSymbolSpin(pNxCoreSys, pNxCoreMsg);
		  break;
	  }

	  // Continue running the tape
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}


	// OnNxCoreStatus: Function to handle NxCore Status messages
	//----------------------------------------------------------
	static unsafe void OnNxCoreStatus(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{

	  // If a Minute has elapsed print the NxCore system time.
	  // NOTE: The last time sent is 24:00:00. Don't print that
	  // time as DateTime() will fail...24:00:00 is not actually
	  // a real time value.
	  if ((pNxCoreSys->ClockUpdateInterval >= NxCore.NxCLOCK_MINUTE) &&
		  (pNxCoreSys->nxTime.Hour < 24))
	  {
		DateTime thisTime = new DateTime(pNxCoreSys->nxDate.Year,
										 pNxCoreSys->nxDate.Month,
										 pNxCoreSys->nxDate.Day,
										 pNxCoreSys->nxTime.Hour,
										 pNxCoreSys->nxTime.Minute,
										 pNxCoreSys->nxTime.Second);

		Console.WriteLine("NxCore Time: {0:T}", thisTime);
	  }
	  
	  // Print the specific NxCore status message
	  switch (pNxCoreSys->Status)
	  {
		case NxCore.NxCORESTATUS_COMPLETE:
		     Console.WriteLine("NxCore Complete Message.");
		     break;

		case NxCore.NxCORESTATUS_INITIALIZING:
		     Console.WriteLine("NxCore Initialize Message.");
		     break;

		case NxCore.NxCORESTATUS_SYNCHRONIZING:
		     Console.WriteLine("NxCore Synchronizing Message.");
		     break;

		case NxCore.NxCORESTATUS_WAITFORCOREACCESS:
		     Console.WriteLine("NxCore Wait For Access.");
		     break;

		case NxCore.NxCORESTATUS_RESTARTING_TAPE:
		     Console.WriteLine("NxCore Restart Tape Message.");
		     break;

		case NxCore.NxCORESTATUS_ERROR:
		     Console.WriteLine("NxCore Error.");
		     break;

		case NxCore.NxCORESTATUS_RUNNING:
		     break;
	  }	  
	}


	// OnNxCoreSymbolSpin: Function to handle NxCore Symbol Spin messages
	//-------------------------------------------------------------------
	static unsafe void OnNxCoreSymbolSpin(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{
	  // Get the primary symbol
	  
	  String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);
	  	  

	  // If this is an option, and there is a valid option header, use it
	  if ((Symbol[0]=='o')&&(pNxCoreMsg->coreHeader.pnxOptionHdr != null))
	  {
		// Get the strike price from header info
		double StrikePrice;
		StrikePrice = pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice / 1000.0;

		// Get the root option symbol. Take out 'o' from root (String+1)	
		String OptionSymbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String + 1);

		// Get the date-strike string
		String DateStrikeString = new String(&pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String);

		String SeriesString="";
		String UnderlyingString="";
		
		// Get the Series string and Underlying symbol string
		// NOTE: VERY IMPORTANT TO NOT PASS NULL POINTERS IN AS ADDY FOR A STRING!!
		// SOME OF THESE STRINGS MAY OCCASIONALLY BE NULL 		
  	    if (pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsSeriesChain != null)
		    SeriesString = new String(&pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsSeriesChain->String);
		if (pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsUnderlying!=null)
		    UnderlyingString = new String(&pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsUnderlying->String);
		
		// If pnxsDateAndStrike->String[1] == ' ', then this symbol is in new OSI format. 		
		if (DateStrikeString[1] == ' ') 										
		{		
		  // Construct OSI Symbol
		  OptionSymbol=String.Format("{0:S}{1:00}{2:00}{3:00}{4}{5:00000000}",
					OptionSymbol.PadRight(6,' '),
					pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Year - 2000,
					pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Month,
					pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Day,
					(pNxCoreMsg->coreHeader.pnxOptionHdr->PutCall == 0) ? 'C' : 'P',
					pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice);

		  Console.WriteLine("Symbol Spin for: NxSym: {0}  OSI Sym: {1}  Strike: {2:F}  Underlying: {3}  Series Chain: {4}",
				  Symbol, OptionSymbol, StrikePrice, UnderlyingString, SeriesString);
		}

		// Otherwise the symbol is in old OPRA format (series+datestrike).		
		else
		{		  
          OptionSymbol+=DateStrikeString;

		  // Print the option symbol (series + datestrike code), underlying symbol and the series chain
		  Console.WriteLine("Symbol Spin for: NxSym: {0}  OPRA Sym: {1}  Strike: {2:F}  Underlying: {3}  Series Chain: {4}",
				  Symbol,OptionSymbol, StrikePrice, UnderlyingString, SeriesString);
		}
								
	  }

	  // Otherwise this is another equity type (stock, future, etc).
	  else 
	  {
		// Print the symbol and listed exchange for this spin message
		Console.WriteLine("Symbol Spin for: {0:S}  Listed Exg: {1:D}",
						  Symbol,
						  pNxCoreMsg->coreHeader.ListedExg);
	  }
	}
		
  }
}
