////////////////////////////////////////////////////
//// SampleApp3.cs                              ////
//// Sample App for C# NxCore API Wrapper       ////
//// Author: Jeffrey Donovan                    ////
//// Date: 12-22-09                             ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// Handling the NxCore Category Message       ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;

// Use the NxCoreAPI namespace
using NxCoreAPI;


namespace SampleApp3
{ 

  class Program
  {
	// Main Entry Point for app.	
	//--------------------------
	static void Main(string[] args)
	{
	  Console.WriteLine("NxCore CSharp SampleApp3 Start.");

	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. Use control flags
	  // to eliminate Level1 quotes, Level2 quotes and all OPRA
	  // quotes.
	  if (args.Length > 0)
		NxCore.ProcessTape(args[0],
						   null,
						   (uint)(NxCore.NxCF_EXCLUDE_QUOTES | NxCore.NxCF_EXCLUDE_QUOTES2 | NxCore.NxCF_EXCLUDE_OPRA),
						   0,
						   Program.OnNxCoreCallback);

	  // Otherwise start NxCore with the current day's tape (IE Realtime)
	  else
		NxCore.ProcessTape("",
						   null,
						   (uint)(NxCore.NxCF_EXCLUDE_QUOTES | NxCore.NxCF_EXCLUDE_QUOTES2 | NxCore.NxCF_EXCLUDE_OPRA),
						   0,
						   Program.OnNxCoreCallback);


	  Console.WriteLine("NxCore CSharp SampleApp3 Stop.");
	}


	// The NXCore Callback Function	
	//-----------------------------
	static unsafe int OnNxCoreCallback(IntPtr pSys, IntPtr pMsg)
	{
	  // Alias structure pointers to the pointers passed in.
	  NxCoreSystem* pNxCoreSys = (NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg = (NxCoreMessage*)pMsg;

	  // Do something based on the message type
	  switch (pNxCoreMsg->MessageType)
	  {
		// NxCore Status Message
		case NxCore.NxMSG_STATUS:
		  OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		  break;

		// NxCore Category Message
		case NxCore.NxMSG_CATEGORY:
		  OnNxCoreCategory(pNxCoreSys, pNxCoreMsg);
		  break;
	  }

	  // Continue running the tape
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}


	// OnNxCoreStatus: Function to handle NxCore Status messages
	//----------------------------------------------------------
	static unsafe void OnNxCoreStatus(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{

	  // If a Minute has elapsed print the NxCore system time.
	  // NOTE: The last time sent is 24:00:00. Don't print that
	  // time as DateTime() will fail...24:00:00 is not actually
	  // a real time value.
	  if ((pNxCoreSys->ClockUpdateInterval >= NxCore.NxCLOCK_MINUTE) &&
		  (pNxCoreSys->nxTime.Hour < 24))
	  {
		DateTime thisTime = new DateTime(pNxCoreSys->nxDate.Year,
										 pNxCoreSys->nxDate.Month,
										 pNxCoreSys->nxDate.Day,
										 pNxCoreSys->nxTime.Hour,
										 pNxCoreSys->nxTime.Minute,
										 pNxCoreSys->nxTime.Second);

		Console.WriteLine("NxCore Time: {0:T}", thisTime);
	  }
	  
	  // Print the specific NxCore status message
	  switch (pNxCoreSys->Status)
	  {
		case NxCore.NxCORESTATUS_COMPLETE:
		     Console.WriteLine("NxCore Complete Message.");
		     break;

		case NxCore.NxCORESTATUS_INITIALIZING:
		     Console.WriteLine("NxCore Initialize Message.");
		     break;

		case NxCore.NxCORESTATUS_SYNCHRONIZING:
		     Console.WriteLine("NxCore Synchronizing Message.");
		     break;

		case NxCore.NxCORESTATUS_WAITFORCOREACCESS:
		     Console.WriteLine("NxCore Wait For Access.");
		     break;

		case NxCore.NxCORESTATUS_RESTARTING_TAPE:
		     Console.WriteLine("NxCore Restart Tape Message.");
		     break;

		case NxCore.NxCORESTATUS_ERROR:
		     Console.WriteLine("NxCore Error.");
		     break;

		case NxCore.NxCORESTATUS_RUNNING:
		     break;
	  }	  
	}
	
	
	// OnNxCoreCategory: Function to handle NxCore Category messages.
	// In this example we only capture category 4 and category 13 messages.
	//--------------------------------------------------------------
	static unsafe void OnNxCoreCategory(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{
	  // Category field pointer
	  NxCategoryField* pField;

	  // Get the symbol for category message
	  String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);
	  	  	  
	  // Do something based on the category num
	  switch (pNxCoreMsg->coreData.Category.pnxStringCategory->Atom)
	  {
		 
		// Category 4, Equity Symbol Information
		case 4:

		  // Grab a pointer to the field
		  pField = &pNxCoreMsg->coreData.Category.pnxFields[0];

		  // Is field set?
		  if (pField->Set == 1)
		  {			
			// Get the Company Name
			String Name = new String(pField->data.StringZ);

			Console.WriteLine("Category 4 for Symbol: {0:S}, Company Name= {1:S}",
							  Symbol, Name);
		  }		  
		  break;


		// Category 13, 52 Week High/Low
		case 13:
		  double High52 = 0.0, Low52 = 0.0;
		  NxDate HighDate=new NxDate();
		  NxDate LowDate= new NxDate(); 
		  
		  // Grab a pointer to the 52 week high date
		  pField = &pNxCoreMsg->coreData.Category.pnxFields[0];
		  if (pField->Set == 1)
			HighDate = pField->data.nxDate;
		  
		  // Grab a pointer to the 52 week low date
		  pField = &pNxCoreMsg->coreData.Category.pnxFields[1];
		  if (pField->Set == 1)
			LowDate = pField->data.nxDate;
		  
		  // Grab a pointer to the 52 week high field
		  pField = &pNxCoreMsg->coreData.Category.pnxFields[2];

		  // Get the 52 week high
		  if (pField->Set == 1)
			High52 = NxCore.PriceToDouble(pField->data.nxPrice.Price, pField->data.nxPrice.PriceType);

		  // Grab a pointer to the 52 week low field
		  pField = &pNxCoreMsg->coreData.Category.pnxFields[3];

		  // Get the 52 week low
		  if (pField->Set == 1)
			Low52 = NxCore.PriceToDouble(pField->data.nxPrice.Price, pField->data.nxPrice.PriceType);

		  Console.WriteLine("Category 13 for Symbol: {0:S}, {1:d2}/{2:d2}/{3:d} High52= {4:f}, {5:d2}/{6:d2}/{7:d} Low52= {8:f}",
							Symbol, 
							HighDate.Month,HighDate.Day,HighDate.Year,High52,
							LowDate.Month, LowDate.Day, LowDate.Year,Low52);
		  break;
	  }	  
	}
						
  }
}
