////////////////////////////////////////////////////
//// SampleApp6.cs                              ////
//// Sample App for C# NxCore API Wrapper       ////
//// Author: Jeffrey Donovan                    ////
//// Date: 01-03-10                             ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// Retriving StateTrades, StateExgQuotes      ////
//// and StateMMQuotes                          ////
////                                            ////
//// NOTE: In this sample, we retrieve State    ////
////       items when NxCore delivers real time ////
////       Trade, Quote and MMQuote messages.   ////
////       This is NOT the most effective way   ////
////       to get the streaming data (see       ////
////       SampleX). However it does effectively////
////       demonstrate retrieving the State for ////
////       any symbol.                          ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

// Use the NxCoreAPI namespace
using NxCoreAPI;


namespace SampleApp6
{ 

  class Program
  {

	// Main Entry Point for app.	
	//--------------------------
	static void Main(string[] args)
	{
	  Console.WriteLine("NxCore CSharp SampleApp6 Start.");

	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  if (args.Length > 0)
		NxCore.ProcessTape(args[0],
						   null, 0, 0,
						   Program.OnNxCoreCallback);

	  // Otherwise start NxCore with the current day's tape (IE Realtime)
	  else
		NxCore.ProcessTape("",
						   null, 0, 0,
						   Program.OnNxCoreCallback);


	  Console.WriteLine("NxCore CSharp SampleApp6 Stop.");
	}


	// The NXCore Callback Function	
	//-----------------------------
	static unsafe int OnNxCoreCallback(IntPtr pSys, IntPtr pMsg)
	{
	  // Alias structure pointers to the pointers passed in.
	  NxCoreSystem* pNxCoreSys = (NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg = (NxCoreMessage*)pMsg;

	  // Do something based on the message type
	  switch (pNxCoreMsg->MessageType)
	  {
		// NxCore Status Message
		case NxCore.NxMSG_STATUS:
		  OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		  break;

		// On a NxCore Trade Message, retreive the Trade State as it 
		// is currently for the issue.
		case NxCore.NxMSG_TRADE:

		  OnGetLastTradeStates(pNxCoreSys, pNxCoreMsg);
		  break;

		// On a NxCore Level1 Quote Message, retreive the ExgQuote State as it 
		// is currently for the issue.
		case NxCore.NxMSG_EXGQUOTE:

		  OnGetExgQuoteStates(pNxCoreSys, pNxCoreMsg);
		  break;

		// On a NxCore Level2 Quote Message, retreive the MMQuote State as it 
		// is currently for the issue.
		case NxCore.NxMSG_MMQUOTE:

		  OnGetMMStates(pNxCoreSys, pNxCoreMsg);
		  break;
	  }

	  // Continue running the tape
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}


	// OnNxCoreStatus: Function to handle NxCore Status messages
	//----------------------------------------------------------
	static unsafe void OnNxCoreStatus(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{

	  // If a Minute has elapsed print the NxCore system time.
	  // NOTE: The last time sent is 24:00:00. Don't print that
	  // time as DateTime() will fail...24:00:00 is not actually
	  // a real time value.
	  if ((pNxCoreSys->ClockUpdateInterval >= NxCore.NxCLOCK_MINUTE) &&
		  (pNxCoreSys->nxTime.Hour < 24))
	  {
		DateTime thisTime = new DateTime(pNxCoreSys->nxDate.Year,
										 pNxCoreSys->nxDate.Month,
										 pNxCoreSys->nxDate.Day,
										 pNxCoreSys->nxTime.Hour,
										 pNxCoreSys->nxTime.Minute,
										 pNxCoreSys->nxTime.Second);

		Console.WriteLine("NxCore Time: {0:T}", thisTime);
	  }
	  
	  // Print the specific NxCore status message
	  switch (pNxCoreSys->Status)
	  {
		case NxCore.NxCORESTATUS_COMPLETE:
		     Console.WriteLine("NxCore Complete Message.");
		     break;

		case NxCore.NxCORESTATUS_INITIALIZING:
		     Console.WriteLine("NxCore Initialize Message.");
		     break;

		case NxCore.NxCORESTATUS_SYNCHRONIZING:
		     Console.WriteLine("NxCore Synchronizing Message.");
		     break;

		case NxCore.NxCORESTATUS_WAITFORCOREACCESS:
		     Console.WriteLine("NxCore Wait For Access.");
		     break;

		case NxCore.NxCORESTATUS_RESTARTING_TAPE:
		     Console.WriteLine("NxCore Restart Tape Message.");
		     break;

		case NxCore.NxCORESTATUS_ERROR:
		     Console.WriteLine("NxCore Error.");
		     break;

		case NxCore.NxCORESTATUS_RUNNING:
		     break;
	  }	  
	}


	// OnGetMMStates: Function to retrieve the current Market
	// maker list for the issue, populated to the current time.
	//----------------------------------------------------------
    static unsafe void OnGetMMStates(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
    {
      String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);
	  
	  NxCoreStateMMQuotes pStatMMQuotes = new NxCoreStateMMQuotes();

	  // Retrieve MM Quote....make sure a valid return from function
	  if (NxCore.StateGetMMQuotes((ushort)pNxCoreMsg->coreHeader.ListedExg,
					               ref pStatMMQuotes,
							       (System.IntPtr)pNxCoreMsg->coreHeader.pnxStringSymbol)==0)
	  {
	    Console.WriteLine("StateMMQuotes for {0:S}, Count={1:d}",
							  Symbol, pStatMMQuotes.StateQuoteCount);

	    // Cycle through all quotes in the state
	    for (int loop = 0; loop < pStatMMQuotes.StateQuoteCount; loop++)
	    {
	      String NewMM = new String(&pStatMMQuotes.StateMMQuotes[loop].pnxStringMarketMaker->String);
	      // Get bid and ask price
	  	  double Bid = NxCore.PriceToDouble(pStatMMQuotes.StateMMQuotes[loop].BidPrice, pStatMMQuotes.PriceType);
		  double Ask = NxCore.PriceToDouble(pStatMMQuotes.StateMMQuotes[loop].AskPrice, pStatMMQuotes.PriceType);
		  int BidSize = pStatMMQuotes.StateMMQuotes[loop].BidSize;
		  int AskSize = pStatMMQuotes.StateMMQuotes[loop].AskSize;

		  Console.WriteLine("Market Maker: {0:S}  Bid:{1:f}  Ask:{2:f}  AskSize:{3:d}  BidSize:{4:d}",
							NewMM, Bid, Ask, BidSize, AskSize);
 	    }	
      }        
    }


	// OnGetExgQuoteStates: Function to retrieve the current Exg quote
	// list for the issue, populated to the current time.
	//----------------------------------------------------------
	static unsafe void OnGetExgQuoteStates(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{
	  String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);
	  
	  System.IntPtr RequestString;

	  // If an option....
	  if ((Symbol[0] == 'o') && (pNxCoreMsg->coreHeader.pnxOptionHdr != null))
		RequestString = (System.IntPtr)pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike;

	  // otherwise a non-option
	  else
		RequestString = (System.IntPtr)pNxCoreMsg->coreHeader.pnxStringSymbol;
	  
	  NxCoreStateExgQuotes pStatExgQuotes = new NxCoreStateExgQuotes();

	  // Retrieve Exg Quote....make sure a valid return from function
	  if (NxCore.StateGetExgQuotes(ref pStatExgQuotes,RequestString)==0)
	  {

	    Console.WriteLine("StateExgQuotes for {0:S}  BestBidExg:{1:d}  BestAskExg:{2:d}",
	  				      Symbol, pStatExgQuotes.BestBidExg, pStatExgQuotes.BestAskExg);

	    // Cycle through all quotes in the state
		for (int loop = 0; loop < pStatExgQuotes.StateQuoteCount; loop++)
		{
		  // Get bid and ask price
		  double Bid = NxCore.PriceToDouble(pStatExgQuotes.StateExgQuotes[loop].BidPrice, pStatExgQuotes.PriceType);
		  double Ask = NxCore.PriceToDouble(pStatExgQuotes.StateExgQuotes[loop].AskPrice, pStatExgQuotes.PriceType);
		  int BidSize = pStatExgQuotes.StateExgQuotes[loop].BidSize;
		  int AskSize = pStatExgQuotes.StateExgQuotes[loop].AskSize;
		  ushort ReportingExg = pStatExgQuotes.StateExgQuotes[loop].ReportingExg;
		  byte Condition = pStatExgQuotes.StateExgQuotes[loop].QuoteCondition;

		  Console.WriteLine("Exg: {0:d}  Bid:{1:f}  Ask:{2:f}  AskSize:{3:d}  BidSize:{4:d}  Condition:{5:d}",
							ReportingExg, Bid, Ask, BidSize, AskSize, Condition);
		}
	  }
	}


	// OnGetLastTradeStates: Function to retrieve the current composite trade
	// info for the issue, populated to the current time.
	//----------------------------------------------------------
	static unsafe void OnGetLastTradeStates(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{
	  String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);

	  System.IntPtr RequestString;

	  // If an option....
	  if ((Symbol[0] == 'o') && (pNxCoreMsg->coreHeader.pnxOptionHdr != null))
		RequestString = (System.IntPtr)pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike;

	  // otherwise a non-option
	  else
		RequestString = (System.IntPtr)pNxCoreMsg->coreHeader.pnxStringSymbol;
	  	  
	  NxCoreStateTrade pStatLastTrade = new NxCoreStateTrade();

	  // Retrieve Last Trade State....make sure a valid return from function
	  if (NxCore.StateGetLastTrade(ref pStatLastTrade, RequestString) == 0)
	  {

		Console.WriteLine("StateLastTrade for {0:S}  Open:{1:f}  High:{2:f}  Low:{3:f}  Last:{4:f}  Price:{5:f}  NetChg:{6:f}  TotalVol:{7:d}",
						  Symbol,
						  NxCore.PriceToDouble(pStatLastTrade.Open, pStatLastTrade.PriceType),
						  NxCore.PriceToDouble(pStatLastTrade.High, pStatLastTrade.PriceType),
						  NxCore.PriceToDouble(pStatLastTrade.Low, pStatLastTrade.PriceType),
						  NxCore.PriceToDouble(pStatLastTrade.Last, pStatLastTrade.PriceType),
						  NxCore.PriceToDouble(pStatLastTrade.Price, pStatLastTrade.PriceType),
						  NxCore.PriceToDouble(pStatLastTrade.NetChange, pStatLastTrade.PriceType),
						  pStatLastTrade.TotalVolume);
	  }
	}			
  }
}
