////////////////////////////////////////////////////
//// SampleApp7.cs                              ////
//// Sample App for C# NxCore API Wrapper       ////
//// Author: Jeffrey Donovan                    ////
//// Date: 01-04-10                             ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// 1) User defined symbol spin function       ////
//// 2) Available NxCore Data file listings     ////
//// 3) Available NxCore DLL file listings      ////
//// 4) Retrieving strings from NxCore Tables   ////
//// 5) Get Date from N days                    ////
//// 6) Get N days from date                    ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

// Use the NxCoreAPI namespace
using NxCoreAPI;

namespace SampleApp7
{ 
  // The program!
  class Program
  {

	// Main Entry Point for app.	
	//--------------------------
	static void Main(string[] args)
	{
	  Console.WriteLine("NxCore CSharp SampleApp7 Start.");

	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  if (args.Length > 0)
		NxCore.ProcessTape(args[0],
						   null,
						   0,
						   0,
						   Program.OnNxCoreCallback);

	  // Otherwise start NxCore with the current day's tape (IE Realtime)
	  else
		NxCore.ProcessTape("",
						   null,
						   0,
						   0,
						   Program.OnNxCoreCallback);


	  Console.WriteLine("NxCore CSharp SampleApp7 Stop.");
	}

	
	// The NXCore Callback Function	
	//-----------------------------
	static unsafe int OnNxCoreCallback(IntPtr pSys, IntPtr pMsg)
	{
	  // Alias structure pointers to the pointers passed in.
	  NxCoreSystem* pNxCoreSys = (NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg = (NxCoreMessage*)pMsg;

	  // Do something based on the message type
	  switch (pNxCoreMsg->MessageType)
	  {

		// NxCore Status Message
		case NxCore.NxMSG_STATUS:
		  OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		  break;
	  }

	  // Continue running the tape
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}


	// OnNxCoreStatus: Function to handle NxCore Status messages
	//----------------------------------------------------------
	static unsafe void OnNxCoreStatus(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{

	  // If a Minute has elapsed print the NxCore system time.
	  // NOTE: The last time sent is 24:00:00. Don't print that
	  // time as DateTime() will fail...24:00:00 is not actually
	  // a real time value.
	  if ((pNxCoreSys->ClockUpdateInterval >= NxCore.NxCLOCK_MINUTE) &&
		  (pNxCoreSys->nxTime.Hour < 24))
	  {
		DateTime thisTime = new DateTime(pNxCoreSys->nxDate.Year,
										 pNxCoreSys->nxDate.Month,
										 pNxCoreSys->nxDate.Day,
										 pNxCoreSys->nxTime.Hour,
										 pNxCoreSys->nxTime.Minute,
										 pNxCoreSys->nxTime.Second);

		Console.WriteLine("NxCore Time: {0:T}", thisTime);
	  }

	  // Print the specific NxCore status message
	  switch (pNxCoreSys->Status)
	  {
		// Capture a SYMBOL SPIN Message
		case NxCore.NxCORESTATUS_SYMBOLSPIN:
		  switch (pNxCoreSys->StatusData)
		  {

			// If initial symbol spin is complete, test the functions in this
			// sample application 
			case NxCore.NxCSSYMBOLSPIN_COMPLETE:

			  // User defined symbol spin
			  Console.WriteLine("NxCore Initial Spin Complete Message.");
			  Console.WriteLine("");
			  Console.WriteLine("Press <Enter> to run a user define spin.");
			  Console.ReadKey();

			  NxCore.SpinSymbols((ushort)NxCore.NxSPIN_ALL_EXGS, 0, 1, Program.OnUserSymbolSpin, 0);

			  // Get defined string from NxCore tables
			  Console.WriteLine("");
			  Console.WriteLine("UserDefine Spin Complete!");
			  Console.WriteLine("");
			  Console.WriteLine("Press <Enter> to get defined strings for exchanges.");
			  Console.ReadKey();

			  GetDefinedString();

			  // Get current tape filename
			  Console.WriteLine("");
			  Console.WriteLine("Press <Enter> to get the Tape Name.");
			  Console.ReadKey();

			  GetTapeName();

			  // List available tapes on system
			  Console.WriteLine("");
			  Console.WriteLine("Press <Enter> to list Tape Files Available.");
			  Console.ReadKey();

			  NxCore.ListTapes((int)NxCore.NxLTF_SEARCH_LOCAL, OnUserListTapes, null);

			  // List available DLL's on system
			  Console.WriteLine("");
			  Console.WriteLine("Press <Enter> to list DLL Files Available.");
			  Console.ReadKey();

			  NxCore.ListAPIDLLs((int)NxCore.NxLTF_SEARCH_LOCAL, OnUserListAPIDLLs, null);

			  // Get date from N days
			  Console.WriteLine("");
			  Console.WriteLine("Press <Enter> to get a date from N days.");
			  Console.ReadKey();

			  DateFromDays();

			  // Get N Days from date
			  Console.WriteLine("");
			  Console.WriteLine("Press <Enter> to get NDays from a date.");
			  Console.ReadKey();

			  DaysFromDate();

			  // Finish!
			  Console.WriteLine("");
			  Console.WriteLine("Press <Enter> to continue with tape.");
			  Console.ReadKey();
			  break;
		  }
		  break;

		case NxCore.NxCORESTATUS_COMPLETE:
		  Console.WriteLine("NxCore Complete Message.");
		  break;

		case NxCore.NxCORESTATUS_INITIALIZING:
		  Console.WriteLine("NxCore Initialize Message.");
		  break;

		case NxCore.NxCORESTATUS_SYNCHRONIZING:
		  Console.WriteLine("NxCore Synchronizing Message.");
		  break;

		case NxCore.NxCORESTATUS_WAITFORCOREACCESS:
		  Console.WriteLine("NxCore Wait For Access.");
		  break;

		case NxCore.NxCORESTATUS_RESTARTING_TAPE:
		  Console.WriteLine("NxCore Restart Tape Message.");
		  break;

		case NxCore.NxCORESTATUS_ERROR:
		  Console.WriteLine("NxCore Error.");
		  break;

		case NxCore.NxCORESTATUS_RUNNING:
		  break;
	  }
	}
	

	// GetDefinedString: Get string description from NxCore table definitions.
	//------------------------------------------------------------------------
	static unsafe void GetDefinedString()
	{
	  // Loop for 60 exchanges...
	  for (int loop = 1; loop < 60; loop++)
	  {
		// Get the defined name for this exchange
		String Exg = new String(NxCore.GetDefinedString((int)NxCore.NxST_EXCHANGE, loop));		

		// Print exchange num and name
		Console.WriteLine("Exg #{0:d} = {1:S}", loop,Exg);
	  }
	}


	// GetTapeName: Get filename for current tape
	//-------------------------------------------
	static unsafe void GetTapeName()
	{
	  char[] str = new char[255];	  	  
	  
	  // Get current tape name, convert to string type
	  NxCore.GetTapeName(str,254);
	  String str2 = new String(str);

	  // Print tape name
	  Console.WriteLine("TapeName {0:S}",str2);	  	  
	}


	// DateFromDays: Get date from N Days
	//-----------------------------------
	static unsafe void DateFromDays()
	{
	  NxDate Date=new NxDate();
	  
	  // Set NDays to get date for
	  Date.NDays=32000;

	  // Call DateFromNDays function
	  NxCore.DateFromNDays(ref Date);

	  // print NDays and date
	  Console.WriteLine("Date for {0:d} Days: {1:d}/{2:d}/{3:d}",
					    Date.NDays,
						Date.Month,
						Date.Day,
						Date.Year);
	}


	// DaysFromDate: Get N days from date
	//-----------------------------------
	static unsafe void DaysFromDate()
	{
	  NxDate Date = new NxDate();

	  // Set date to get NDays from
	  Date.Month=1;
  	  Date.Day  =1;
	  Date.Year = 2010;

	  // Call DateFromYMD function
	  NxCore.DateFromYMD(ref Date);

	  // Print date and NDays
	  Console.WriteLine("Date {0:d}/{1:d}/{2:d} = {3:d} Days",						
						Date.Month,
						Date.Day,
						Date.Year,
						Date.NDays);
	}


	// OnUserListTapes: Callback with abailable tape filenames on sytstem
	//-------------------------------------------------------------------
	static unsafe int OnUserListTapes(IntPtr pUser, IntPtr pTapeFile)
	{
	  // Get the tape filename and print it
	  NxCoreTapeFile* TapeFile = (NxCoreTapeFile*)pTapeFile;
	  String Filename = new String(&TapeFile->PathnameStrZ);
	  Console.WriteLine("Filename: {0:S}", Filename);
	  
	  // Continue to next
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}


	// OnUserListAPIDLLs: Callback with abailable NxCore DLL's on sytstem
	//-------------------------------------------------------------------
	static unsafe int OnUserListAPIDLLs(IntPtr pUser, IntPtr pDLLFile)
	{
	  // Get the DLL filename and print it
	  NxCoreAPIDLLFile* DLLFile = (NxCoreAPIDLLFile*)pDLLFile;
	  String Filename = new String(&DLLFile->PathnameStrZ);
	  Console.WriteLine("Filename: {0:S}", Filename);

	  // Contiue to next
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}


	// OnUserSymbolSpin:  User defined symbol spin callback
	//------------------------------------------------------
	static unsafe int OnUserSymbolSpin(IntPtr pSys, IntPtr pMsg)
	{
	  // Grab proper NxCore pointers
	  NxCoreSystem* pNxCoreSys=(NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg=(NxCoreMessage*)pMsg;

	  // Get the symbol 
	  String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);

	  
	  // Print the spin message sym, exg, time, date and session ID	  
	  Console.WriteLine("UserSymbol Spin for: {0:S}  Listed Exg: {1:d}  Time: {2:d2}:{3:d2}:{4:d2} {5:d2}/{6:d2}/{7:d}  {8:d}",
						Symbol,
						pNxCoreMsg->coreHeader.ListedExg,
						pNxCoreMsg->coreHeader.nxExgTimestamp.Hour,
						pNxCoreMsg->coreHeader.nxExgTimestamp.Minute,
						pNxCoreMsg->coreHeader.nxExgTimestamp.Second,
						pNxCoreMsg->coreHeader.nxSessionDate.Month,
						pNxCoreMsg->coreHeader.nxSessionDate.Day,
						pNxCoreMsg->coreHeader.nxSessionDate.Year,
						pNxCoreMsg->coreHeader.SessionID);
	  
	  // Continue to next
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}
		
  }
}
