//////////////////////////////////////////////////////
//// SampleApp8.cs                                ////
//// Sample App for C# NxCore API Wrapper         ////
//// Author: Jeffrey Donovan                      ////
//// Date: 01-09-10                               ////
//////////////////////////////////////////////////////
//// Demonstrates:                                ////
//// 1) User defined symbol spin function         ////
//// 2) Setting UserData1 and UserData2 values    ////
//// 3) Retrieving UserData1 and UserData2 values ////
//// 4) Determining if a tape is OSI from the tape////
////    version number.                           ////
//// 5) Constructing Option Symbols in OSI or     ////
////    old OPRA format                           ////
//////////////////////////////////////////////////////
//// To Read as Written:                          ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs       ////
//////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

// Use the NxCoreAPI namespace
using NxCoreAPI;

namespace SampleApp8
{ 
  // The program!
  class Program
  {
	// We'll use these to set the UserData1 and UserData1 members of NxString in the
	// initial symbol spin, and retrieve the values from two UserDefined Spins
	static Int32 MainCounter = 0;
	static Int32 MainOptionCounter = 0;

	static bool IsOSI=false;


	// Main Entry Point for app.	
	//--------------------------
	static void Main(string[] args)
	{
	  Console.WriteLine("NxCore CSharp SampleApp8 Start.");

	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  if (args.Length > 0)
		NxCore.ProcessTape(args[0],
						   null,0,0,
						   Program.OnNxCoreCallback);

	  // Otherwise start NxCore with the current day's tape (IE Realtime)
	  else
		NxCore.ProcessTape("",
						   null,0,0,
						   Program.OnNxCoreCallback);


	  Console.WriteLine("NxCore CSharp SampleApp8 Stop.");
	}


	// The NXCore Callback Function	
	//-----------------------------
	static unsafe int OnNxCoreCallback(IntPtr pSys, IntPtr pMsg)
	{
	  // Alias structure pointers to the pointers passed in.
	  NxCoreSystem* pNxCoreSys = (NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg = (NxCoreMessage*)pMsg;

	  // Do something based on the message type
	  switch (pNxCoreMsg->MessageType)
	  {
		// NxCore Symbol Spin message
		case NxCore.NxMSG_SYMBOLSPIN:
		  OnNxCoreSymbolSpin(pNxCoreSys, pNxCoreMsg);
		  break;

		// NxCore Status Message
		case NxCore.NxMSG_STATUS:
		  OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		  break;
	  }

	  // Continue running the tape
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}


	// ExtractTapeVersion  - Extract Tape Version Number
	//----------------------------------------------------------
	static unsafe void ExtractTapeVersion(String TapeString,
							              ref String VersionString,
							              int *Major, int *Minor, int *Sub)
    {	  
	  int pat = 12;	  
	  String Extract;
	  String MajorS, MinorS, SubS;
	  int len = TapeString.Length;
	  int vpos = 0;

	  VersionString=MajorS=MinorS=SubS=Extract = "";

	  while ((pat < len)&&(TapeString[pat]!='\n'))
	  {
		VersionString += TapeString[pat];

		if (TapeString[pat] == '.')
		{
		  if (vpos == 0) MajorS = Extract;
		  else MinorS = Extract;
		  Extract = "";
		  ++vpos;
		}
		else
		{
		  Extract += TapeString[pat];		  
		}
		++pat;
	  }
	  SubS = Extract;

	  *Major = int.Parse(MajorS);
	  *Minor = int.Parse(MinorS);
	  *Sub   = int.Parse(SubS);
    }

	
	// OnNxCoreStatus: Function to handle NxCore Status messages
	//----------------------------------------------------------
	static unsafe void OnNxCoreStatus(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{

	  // If a Minute has elapsed print the NxCore system time.
	  // NOTE: The last time sent is 24:00:00. Don't print that
	  // time as DateTime() will fail...24:00:00 is not actually
	  // a real time value.
	  if ((pNxCoreSys->ClockUpdateInterval >= NxCore.NxCLOCK_MINUTE) &&
		  (pNxCoreSys->nxTime.Hour < 24))
	  {
		DateTime thisTime = new DateTime(pNxCoreSys->nxDate.Year,
										 pNxCoreSys->nxDate.Month,
										 pNxCoreSys->nxDate.Day,
										 pNxCoreSys->nxTime.Hour,
										 pNxCoreSys->nxTime.Minute,
										 pNxCoreSys->nxTime.Second);

		Console.WriteLine("NxCore Time: {0:T}", thisTime);
	  }

	  // Print the specific NxCore status message
	  switch (pNxCoreSys->Status)
	  {
		// SYMBOL SPIN Status Message
		case NxCore.NxCORESTATUS_SYMBOLSPIN:
		  switch (pNxCoreSys->StatusData)
		  {

			// If initial symbol spin is complete, then userdata1 and userdata2 have
			// now been set for all symbols in the initial symbol spin. Call the user
			// defined spins to retrive those values.			
			case NxCore.NxCSSYMBOLSPIN_COMPLETE:

			  // User defined symbol spin for non options
			  Console.WriteLine("NxCore Initial Spin Complete Message.");
			  Console.WriteLine("");
			  Console.WriteLine("Press <Enter> to run a user define spin for non-options.");
			  Console.ReadKey();

			  NxCore.SpinSymbols((ushort)NxCore.NxSPIN_ALL_EXGS, 0, 1, Program.OnUserSymbolSpin_NonOptions, 0);

			  // User defined symbol spin for options
			  Console.WriteLine("NxCore User Spin Complete Message.");
			  Console.WriteLine("");
			  Console.WriteLine("Press <Enter> to run a user define spin for options.");
			  Console.ReadKey();

			  // If the tape is an OSI tape, point the spin to the OSI handler,
			  // otherwise point the spin to the Traditional OPRA handler
			  if (Program.IsOSI)
				NxCore.SpinSymbols((ushort)NxCore.NxSPIN_ALL_EXGS, 1, 1, Program.OnUserSymbolSpin_OptionsOSI, 0);
			  else
			    NxCore.SpinSymbols((ushort)NxCore.NxSPIN_ALL_EXGS, 1, 1, Program.OnUserSymbolSpin_OptionsTraditional, 0);

			  // Continue
			  Console.WriteLine("");
			  Console.WriteLine("Press <Enter> to list continue.");
			  Console.ReadKey();
			  break;
		  }
		  break;

		case NxCore.NxCORESTATUS_COMPLETE:
		  Console.WriteLine("NxCore Complete Message.");
		  break;

		case NxCore.NxCORESTATUS_INITIALIZING:
		  Console.WriteLine("NxCore Initialize Message.");
		  break;

		case NxCore.NxCORESTATUS_SYNCHRONIZING:
		  Console.WriteLine("NxCore Synchronizing Message.");
		  break;

		case NxCore.NxCORESTATUS_WAITFORCOREACCESS:
		  Console.WriteLine("NxCore Wait For Access.");
		  break;

		case NxCore.NxCORESTATUS_RESTARTING_TAPE:
		  Console.WriteLine("NxCore Restart Tape Message.");
		  break;

		case NxCore.NxCORESTATUS_ERROR:
		  Console.WriteLine("NxCore Error.");
		  break;

		case NxCore.NxCORESTATUS_RUNNING:
		  break;

		// Tape Open Message. Get the tape version number
	    case NxCore.NxCORESTATUS_TAPEOPEN:
		  String VString="";
          int Major,Minor,Sub;
		  	   
		  // Extract tape version from TapeDisplay string
		  ExtractTapeVersion(new String(pNxCoreSys->TapeDisplay),ref VString,&Major,&Minor,&Sub);

		  Console.WriteLine("");
		  Console.WriteLine("NxCore Tape Open Message.");
		  Console.WriteLine("Tape Version String: {0} Tape Version Numeric: {1} {2} {3}  System Date {4}/{5}/{6}",
			      VString,Major,Minor,Sub,pNxCoreSys->nxDate.Month,pNxCoreSys->nxDate.Day,pNxCoreSys->nxDate.Year);

		  // If tape version is greater than 1.20.x then this is an OSI tape
	   	  if ((Major>1)||((Major==1)&&(Minor>20))) Program.IsOSI=true;

		  Console.WriteLine("Press <Enter> to Continue...");
		  Console.ReadKey();
		  break;
	  }
	}

	
	// OnNxCoreSymbolSpin: Function to handle Initial NxCore Symbol Spin messages
	// This is were we will SET the UserData1 and UserData2 Values
	//-------------------------------------------------------------------
	static unsafe void OnNxCoreSymbolSpin(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{
	  // Get the primary symbol
	  String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);
	  Int32 UData1 = 0;
	  Int32 UData2 = 0;
	  
	  // If this is an option, and there is a valid option header, use it
	  if ((Symbol[0] == 'o') && (pNxCoreMsg->coreHeader.pnxOptionHdr != null))
	  {		
		// Get the root option symbol. Take out 'o' from root (String+1)	
		String OptionSymbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String + 1);

		// Get the date-strike string
		String DateStrikeString = new String(&pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String);

		// SET UserData1 for the option to MainOptionCounter (increment the counter as we do).
		UData1 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData1 = ++Program.MainOptionCounter;

		// SET UserData2 to some odd value based on Nx Clock
		UData2 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData2 = (pNxCoreSys->nxTime.Second * pNxCoreSys->nxTime.Millisecond);

		// If tape is determined to be an OSI tape...
		if (Program.IsOSI)
		{
		  // Construct OSI Symbol
		  OptionSymbol = String.Format("{0:S}{1:00}{2:00}{3:00}{4}{5:00000000}",
					OptionSymbol.PadRight(6, ' '),
					pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Year - 2000,
					pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Month,
					pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Day,
					(pNxCoreMsg->coreHeader.pnxOptionHdr->PutCall == 0) ? 'C' : 'P',
					pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice);

		  Console.WriteLine("Initial Symbol Spin for: NxSym: {0}  OSI Sym: {1}  UserData1: {2:d}  UserData2: {3:d}",
				  Symbol, OptionSymbol, UData1,UData2);
		}
		// Otherwise the symbol is in old OPRA format (series+datestrike).		
		else
		{
		  OptionSymbol += DateStrikeString;

		  // Print the option symbol (series + datestrike code), underlying symbol and the series chain
		  Console.WriteLine("Initial Symbol Spin for: NxSym: {0}  OPRA Sym: {1}  UserData1: {2:d}  UserData2: {3:d}",
				  Symbol, OptionSymbol, UData1, UData2);
		}
	  }

	  // Otherwise this is another equity type (stock, future, etc).
	  else
	  {
		// SET UserData1 to the MainCounter (increment the counter as we do).        
        UData1 = pNxCoreMsg->coreHeader.pnxStringSymbol->UserData1 = ++Program.MainCounter;

		// SET UserData2 to some odd value based on Nx Clock
		UData2 = pNxCoreMsg->coreHeader.pnxStringSymbol->UserData2 = (pNxCoreSys->nxTime.Second * pNxCoreSys->nxTime.Millisecond);

		// Print the symbol and listed exchange, userdata 1 and 2 for this spin message
		Console.WriteLine("Initial Symbol Spin for: {0:S}  Listed Exg: {1:D}  UserData1: {2:d}  UserData2: {3:d}",
						  Symbol,
						  pNxCoreMsg->coreHeader.ListedExg, UData1, UData2);
	  }
	}

	
	// OnUserSymbolSpin:  User defined symbol spin callback
	//------------------------------------------------------
	static unsafe int OnUserSymbolSpin_NonOptions(IntPtr pSys, IntPtr pMsg)
	{
	  Int32 UData1 = 0;
	  Int32 UData2 = 0;

	  // Grab proper NxCore pointers
	  NxCoreSystem* pNxCoreSys=(NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg=(NxCoreMessage*)pMsg;

	  // Get the symbol 
	  String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);
	  
	  // Retrieve the UseerData1 and UserData2 values set in the initial spin
	  UData1 = (Int32)pNxCoreMsg->coreHeader.pnxStringSymbol->UserData1;
	  UData2 = (Int32)pNxCoreMsg->coreHeader.pnxStringSymbol->UserData2;

	  // Print the spin message sym, exg
	  // NOTE: The last item is pNxCoreMsg->coreHeader.pnxStringSymbol->UserData1 & 2,
	  // which is set in the main symbol spin to a user defined value
	  Console.WriteLine("UserSymbol Spin for: {0:S}  Listed Exg: {1:d}  UserData1: {2:d}  UserData2: {3:d}",
					  Symbol,
					  pNxCoreMsg->coreHeader.ListedExg,
					  UData1, UData2);

	  
	  // Continue to next
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}


	// OnUserSymbolSpin:  User defined symbol spin callback
	//------------------------------------------------------
	static unsafe int OnUserSymbolSpin_OptionsTraditional(IntPtr pSys, IntPtr pMsg)
	{
	  Int32 UData1 = 0;
	  Int32 UData2 = 0;

	  // Grab proper NxCore pointers
	  NxCoreSystem* pNxCoreSys = (NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg = (NxCoreMessage*)pMsg;

	  // Get the symbol 
	  String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);

	  // If this is an option, and there is a valid option header, use it
	  if ((Symbol[0] == 'o') && (pNxCoreMsg->coreHeader.pnxOptionHdr != null))
	  {
		// Get the root option symbol. Take out 'o' from root (String+1)	
		String OptionSymbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String + 1);

		// Get the date-strike string
		String DateStrikeString = new String(&pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String);

		// Retrieve the UserData1 and UserData2 values set in the initial spin
		UData1 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData1;
		UData2 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData2;

	    OptionSymbol += DateStrikeString;

	    // Print the option symbol (series + datestrike code), underlying symbol and the series chain
	    Console.WriteLine("UserSymbol Spin for: NxSym: {0}  OPRA Sym: {1}  UserData1: {2:d}  UserData2: {3:d}",
				  Symbol, OptionSymbol, UData1, UData2);	
	  }

	  // Continue to next
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}

	// OnUserSymbolSpin:  User defined symbol spin callback
	//------------------------------------------------------
	static unsafe int OnUserSymbolSpin_OptionsOSI(IntPtr pSys, IntPtr pMsg)
	{
	  Int32 UData1 = 0;
	  Int32 UData2 = 0;

	  // Grab proper NxCore pointers
	  NxCoreSystem* pNxCoreSys = (NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg = (NxCoreMessage*)pMsg;

	  // Get the symbol 
	  String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);

	  // If this is an option, and there is a valid option header, use it
	  if ((Symbol[0] == 'o') && (pNxCoreMsg->coreHeader.pnxOptionHdr != null))
	  {
		// Get the root option symbol. Take out 'o' from root (String+1)	
		String OptionSymbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String + 1);

		// Get the date-strike string
		String DateStrikeString = new String(&pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String);

		// Retrieve the UserData1 and UserData2 values set in the initial spin
		UData1 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData1;
		UData2 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData2;

	    // Construct OSI Symbol
		OptionSymbol = String.Format("{0:S}{1:00}{2:00}{3:00}{4}{5:00000000}",
					OptionSymbol.PadRight(6, ' '),
					pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Year - 2000,
					pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Month,
					pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Day,
					(pNxCoreMsg->coreHeader.pnxOptionHdr->PutCall == 0) ? 'C' : 'P',
					pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice);

		Console.WriteLine("UserSymbol Spin for: NxSym: {0}  OSI Sym: {1}  UserData1: {2:d}  UserData2: {3:d}",
				  Symbol, OptionSymbol, UData1, UData2);		
	  }	  

	  // Continue to next
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}
  }
}
