//////////////////////////////////////////////////////
//// SampleApp10.cs                               ////
//// Sample App for C# NxCore API Wrapper         ////
//// Author: Jeffrey Donovan                      ////
//// Date: 01-10-10                               ////
//////////////////////////////////////////////////////
//// Demonstrates:                                ////
//// 1) Saving to a NxCore State Tape             ////
//// 2) Running from an NxCore State Tape         ////
//////////////////////////////////////////////////////
//// To Read as Written:                          ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs       ////
//////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

// Use the NxCoreAPI namespace
using NxCoreAPI;

namespace SampleApp10
{ 
 
  // The program!
  class Program
  {
	static Byte SavedState=0;
	static String StateFileFilename="STATESAVE.TMP";
		
	// Main Entry Point for app.	
	//--------------------------
	static void Main(string[] args)
	{
	  Console.WriteLine("NxCore CSharp SampleApp10 Start.");

	  Console.WriteLine("");
	  Console.WriteLine("The application will process trades until 9:00 AM and then save the state,");
	  Console.WriteLine("exit and reload from the state tape.\n\nPress <ENTER> to continue.");
	  Console.ReadKey();

	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  if (args.Length > 0)
		NxCore.ProcessTape(args[0],
						   null,0,0,
						   Program.OnNxCoreCallback);

	  // Otherwise start NxCore with the current day's tape (IE Realtime)
	  else
		NxCore.ProcessTape("",
						   null,0,0,
						   Program.OnNxCoreCallback);

	  // The current process has exited and the state tape has been saved!
	  Console.WriteLine("Tape state saved and process exited.");
	  Console.WriteLine("Press <ENTER> to Load the state and continue.");
	  Console.ReadKey();

	  // Reset this flag so we dont stop in the next call to ProcessTape
	  SavedState=0;

	  // Process the tape from a state file
	  NxCore.ProcessTape(StateFileFilename,
                         null,0,0,
						 Program.OnNxCoreCallback);
	  
	  Console.WriteLine("NxCore CSharp SampleApp10 Stop.");
	}


	// The NXCore Callback Function	
	//-----------------------------
	static unsafe int OnNxCoreCallback(IntPtr pSys, IntPtr pMsg)
	{
	  // If this flag is set, NxCore has saved the state. Note that from the time
	  // the NxCORESTATUS_SAVING_STATE message was received, the function issued
	  // one moreNxCALLBACKRETURN_CONTINUE, and the next time in we will exit, thus
	  // allowing NxCore to properly save the state and to exit automatically.
	  if (Program.SavedState==1)
		return NxCore.NxCALLBACKRETURN_STOP;


	  // Alias structure pointers to the pointers passed in.
	  NxCoreSystem* pNxCoreSys = (NxCoreSystem*)pSys;
	  NxCoreMessage* pNxCoreMsg = (NxCoreMessage*)pMsg;
	  	  
	  // Do something based on the message type
	  switch (pNxCoreMsg->MessageType)
	  {
		// NxCore Trade Message
		case NxCore.NxMSG_TRADE:
		  OnNxCoreTrade(pNxCoreSys, pNxCoreMsg);
		  break;

		// NxCore Status Message
		case NxCore.NxMSG_STATUS:
		  OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		  break;
	  }

	  // Continue running the tape
	  return (int)NxCore.NxCALLBACKRETURN_CONTINUE;
	}

	
	// OnNxCoreStatus: Function to handle NxCore Status messages
	//----------------------------------------------------------
	static unsafe void OnNxCoreStatus(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{

	  // If a Minute has elapsed print the NxCore system time.
	  // NOTE: The last time sent is 24:00:00. Don't print that
	  // time as DateTime() will fail...24:00:00 is not actually
	  // a real time value.
	  if ((pNxCoreSys->ClockUpdateInterval >= NxCore.NxCLOCK_MINUTE) &&
		  (pNxCoreSys->nxTime.Hour < 24))
	  {
		DateTime thisTime = new DateTime(pNxCoreSys->nxDate.Year,
										 pNxCoreSys->nxDate.Month,
										 pNxCoreSys->nxDate.Day,
										 pNxCoreSys->nxTime.Hour,
										 pNxCoreSys->nxTime.Minute,
										 pNxCoreSys->nxTime.Second);

		Console.WriteLine("NxCore Time: {0:T}", thisTime);

		// LOOK! If it's 9:00:00 AM SAVE THE CURRENT STATE TO A STATE FILE!
	    if (pNxCoreSys->nxTime.Hour==9 && pNxCoreSys->nxTime.Minute==0 && pNxCoreSys->nxTime.Second==0)
	    {
		  // NOTE: This application will only work properly with the NxSAVESTATE_ONEPASS flag setting,
		  // as we exit the tape exactly one pass after the file is to be saved. Should you modify this
		  // flag the application will need to also be modified accordingly.
		  NxCore.SaveState(StateFileFilename,NxCore.NxSAVESTATE_ONEPASS);
	    }
	  }

	  // Print the specific NxCore status message
	  switch (pNxCoreSys->Status)
	  {
		// NxCore will save the state this pass. Tell user and wait for keypress
		case NxCore.NxCORESTATUS_SAVING_STATE:
		  Console.WriteLine("");
		  Console.WriteLine("NxCore Will Save the State Now!");
		  Console.WriteLine("Press <ENTER> to Continue.");
		  Console.WriteLine("");
		  Console.ReadKey();

		  // Set the state saved flag
		  SavedState = 1;
		  break;

		// NxCore Has been loaded from a state tape. Tell user and wait for keypress
		case NxCore.NxCORESTATUS_LOADED_STATE:
		  Console.WriteLine("");
		  Console.WriteLine("NxCore Has Been Loaded From A State!");
		  Console.WriteLine("Press <ENTER> to continue processing trades.");
		  Console.ReadKey();
		  break;

		case NxCore.NxCORESTATUS_COMPLETE:
		  Console.WriteLine("NxCore Complete Message.");
		  break;

		case NxCore.NxCORESTATUS_INITIALIZING:
		  Console.WriteLine("NxCore Initialize Message.");
		  break;

		case NxCore.NxCORESTATUS_SYNCHRONIZING:
		  Console.WriteLine("NxCore Synchronizing Message.");
		  break;

		case NxCore.NxCORESTATUS_WAITFORCOREACCESS:
		  Console.WriteLine("NxCore Wait For Access.");
		  break;

		case NxCore.NxCORESTATUS_RESTARTING_TAPE:
		  Console.WriteLine("NxCore Restart Tape Message.");
		  break;

		case NxCore.NxCORESTATUS_ERROR:
		  Console.WriteLine("NxCore Error.");
		  break;

		case NxCore.NxCORESTATUS_RUNNING:
		  break;
	  }
	}


	// OnNxCoreTrade: Function to handle NxCore Trade messages.	
	//--------------------------------------------------------------
	static unsafe void OnNxCoreTrade(NxCoreSystem* pNxCoreSys, NxCoreMessage* pNxCoreMsg)
	{

	  // Get the symbol for category message
	  String Symbol = new String(&pNxCoreMsg->coreHeader.pnxStringSymbol->String);

	  if (Symbol[0] == 'e')
	  {

		// Assign a pointer to the Trade data
		NxCoreTrade* Trade = &pNxCoreMsg->coreData.Trade;

		// Get the price and net change
		double Price = NxCore.PriceToDouble(Trade->Price, Trade->PriceType);
		double NetChange = NxCore.PriceToDouble(Trade->NetChange, Trade->PriceType);

		// Write out Symbol, Time, Price, NetChg, Size, Reporting Exg
		Console.WriteLine("Trade for Symbol: {0:S}, Time: {1:d2}:{2:d2}:{3:d2}  Price: {4:f}  NetChg: {5:f}  Size: {6:d}  Exchg: {7:d} ",
						  Symbol,
						  pNxCoreMsg->coreHeader.nxExgTimestamp.Hour, pNxCoreMsg->coreHeader.nxExgTimestamp.Minute, pNxCoreMsg->coreHeader.nxExgTimestamp.Second,
						  Price, NetChange, Trade->Size,
						  pNxCoreMsg->coreHeader.ReportingExg);
	  }
	}	
  }
}
