////////////////////////////////////////////////////
//// SampleApp1.cpp                             ////
//// Sample App for NxCore in C++               ////
//// Authors: Jeffrey Donovan, Jeremy Truelove  ////
//// Date: 2017-02-08                           ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// 1) Starting NxCore ProcessTape             ////
//// 2) Defining/using NxCore Callback Function ////
//// 3) Handling the NxCore Status Message      ////
////////////////////////////////////////////////////

#include "StdAfx.h"

#include <cstdlib>
#include <iostream>    				// cout, endl
#include <iomanip>					// setw
#include "NxCoreAPI_Wrapper_C++.h"  // NxCore C++ wrapper with NxCoreClass

// Couple forwards - functions are below
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);

// Global Instance of NxCore Class
NxCoreClass NxCore;

// Main Entry Point for app.
int main(int argc, char* argv[])
{
	int returnValue;

	if(argc < 2) {
		cout << "SampleApp01 dll-file [tape-name]" << endl;
		exit(EXIT_SUCCESS);
	}
	cout << "NxCore C++ SampleApp01 Start." << endl;

	// If we can load the NxCore libnx.so
	if (NxCore.LoadNxCore(argv[1]))
	{
		// If a tape filename was passed in command line argument,
		// call ProcessTape with that argument. Use control flags
		// to eliminate Level1 quotes, Level2 quotes and all OPRA
		// quotes.
		if (argv[2]){
			cout << "Processing the tape: " << argv[2] << endl;
			returnValue = NxCore.ProcessTape(argv[2], NULL, (NxCF_EXCLUDE_QUOTES | NxCF_EXCLUDE_QUOTES2 | NxCF_EXCLUDE_OPRA), 0, (NxCoreCallback)OnNxCoreCallback);
		} else {
			returnValue = NxCore.ProcessTape("", NULL, (NxCF_EXCLUDE_QUOTES | NxCF_EXCLUDE_QUOTES2 | NxCF_EXCLUDE_OPRA), 0, (NxCoreCallback)OnNxCoreCallback);
		}

		// output return value from processTape(), helps in troubleshooting potential issues
		NxCore.ProcessReturnValue(returnValue);

		// Unload NxCore DLL
		NxCore.UnloadNxCore();
    }
    else {
	  cout << "Failed to load NxCore Library: " << dlerror() << endl;
	  exit(EXIT_FAILURE);
	}

	cout << "NxCore C++ SampleApp01 Stop." << endl;
	exit(EXIT_SUCCESS);
}


int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  // If this is an NxCore Status message pass it to the status
  // message handler function.
  if (pNxCoreMsg->MessageType == NxMSG_STATUS)
	 OnNxCoreStatus(pNxCoreSys,pNxCoreMsg);

  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;
}

//------------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{

	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
	// a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
	  cout.fill('0');
	  cout << "NxCore Time: " <<
		setw(2) << (int) pNxCoreSys->nxDate.Month << "/" <<
		setw(2) << (int) pNxCoreSys->nxDate.Day << "/" <<
		setw(4) << (int) pNxCoreSys->nxDate.Year << " " <<

		setw(2) << (int) pNxCoreSys->nxTime.Hour << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Minute << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Second << endl;
	}

	// Print the specific NxCore status message
	switch (pNxCoreSys->Status) {
	case NxCORESTATUS_COMPLETE:
		cout << "NxCore Complete Message." << endl;
		break;

	case NxCORESTATUS_INITIALIZING:
		cout << "NxCore Initialize Message." << endl;
		cout << "libnx.so version is v" <<
			NxCore.GetMajorVersion(pNxCoreSys->DLLVersion) << "." <<
			NxCore.GetMinorVersion(pNxCoreSys->DLLVersion) << "." <<
			NxCore.GetBuildVersion(pNxCoreSys->DLLVersion) << endl;
		break;

	case NxCORESTATUS_SYNCHRONIZING:
		cout << "NxCore Synchronizing Message." << endl;
		break;

	case NxCORESTATUS_WAITFORCOREACCESS:
		cout << "NxCore Wait For Access." << endl;
		break;

	case NxCORESTATUS_RESTARTING_TAPE:
		cout << "NxCore Restart Tape Message." << endl;
		break;

	case NxCORESTATUS_ERROR:
		cout << "NxCore Error." << endl;
		break;

	case NxCORESTATUS_RUNNING:
		break;

	}
}
