////////////////////////////////////////////////////
//// SampleApp2.cpp                             ////
//// Sample App for NxCore in C++               ////
//// Authors: Jeffrey Donovan, Jeremy Truelove  ////
//// Date: 2017-02-09                           ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// 1) Handling the NxCore Symbol Spin Message ////
//// 2) Using NxOptionHdr                       ////
//// 3) Constructing Option Symbols in OSI or   ////
////    old OPRA format                         ////
////////////////////////////////////////////////////

#include "StdAfx.h"

#include <cstdlib>
#include <iostream>    				// cout, endl
#include <iomanip>					// setw, fixed, setprecision
#include <stdio.h>					// sprintf
#include "NxCoreAPI_Wrapper_C++.h"  // NxCore C++ wrapper with NxCoreClass

// Couple forwards - functions are below
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreSymbolSpin(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);

// Global instance of NxCore Class
NxCoreClass NxCore;

// Main Entry Point for app.	
int main(int argc, char* argv[])
{   
	int returnValue;

	if(argc < 2) {
		cout << "SampleApp02 dll-file [tape-name]" << endl;
		exit(EXIT_SUCCESS);
	}
    cout << "NxCore C++ SampleApp02 Start. " << endl;

    // If we can load the NxCore libnx.so
	if (NxCore.LoadNxCore(argv[1]))
    {
	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument.
	  if (argv[2])
	  {
		cout << "Processing the tape: " << argv[2] << endl;
	        returnValue = NxCore.ProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
	  }
	  else
		returnValue = NxCore.ProcessTape("", NULL, 0 ,0, (NxCoreCallback)OnNxCoreCallback);

	  // output return value from processTape(), helps in troubleshooting potential issues
	  NxCore.ProcessReturnValue(returnValue);

      // Unload NxCore DLL
	  NxCore.UnloadNxCore();
    }
    else {
	  cout << "Failed to load NxCore Library: " << dlerror() << endl;
	  exit(EXIT_FAILURE);
	}

	cout << "NxCore C++ SampleApp02 Stop." << endl;
	exit(EXIT_SUCCESS);
}


// The NXCore Callback Function	
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{ 
  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
	// NxCore Status Message
	case NxMSG_STATUS:
	     OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		 break;

	// NxCore Symbol Spin Message
	case NxMSG_SYMBOLSPIN:
	     OnNxCoreSymbolSpin(pNxCoreSys, pNxCoreMsg);
		 break;
	  }
	  
  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;  
}


// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
    // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
	  cout.fill('0');
	  cout << "NxCore Time: " <<
		setw(2) << (int) pNxCoreSys->nxDate.Month << "/" <<
		setw(2) << (int) pNxCoreSys->nxDate.Day << "/" <<
		setw(4) << (int) pNxCoreSys->nxDate.Year << " " <<

		setw(2) << (int) pNxCoreSys->nxTime.Hour << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Minute << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Second << endl;
	}
	
	// Print the specific NxCore status message
	switch (pNxCoreSys->Status) {
	case NxCORESTATUS_COMPLETE:
		cout << "NxCore Complete Message." << endl;
		break;

	case NxCORESTATUS_INITIALIZING:
		cout << "NxCore Initialize Message." << endl;
		cout << "libnx.so version is v" <<
			NxCore.GetMajorVersion(pNxCoreSys->DLLVersion) << "." <<
			NxCore.GetMinorVersion(pNxCoreSys->DLLVersion) << "." <<
			NxCore.GetBuildVersion(pNxCoreSys->DLLVersion) << endl;
		break;

	case NxCORESTATUS_SYNCHRONIZING:
		cout << "NxCore Synchronizing Message." << endl;
		break;

	case NxCORESTATUS_WAITFORCOREACCESS:
		cout << "NxCore Wait For Access." << endl;
		break;

	case NxCORESTATUS_RESTARTING_TAPE:
		cout << "NxCore Restart Tape Message." << endl;
		break;

	case NxCORESTATUS_ERROR:
		cout << "NxCore Error." << endl;
		break;

	case NxCORESTATUS_RUNNING:
		break;

	}
}

// OnNxCoreSymbolSpin: Function to handle NxCore Symbol Spin messages
//-------------------------------------------------------------------
void OnNxCoreSymbolSpin(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  // If this is an option, and there is a valid option header, use it
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&(pNxCoreMsg->coreHeader.pnxOptionHdr != NULL))
  {
    // Get the strike price from header info
	double StrikePrice;
	StrikePrice = pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice / 1000.0;

	char OptionSymbol[40];
	char UnderlyingStr[25];
	char SeriesStr[255];	
	UnderlyingStr[0]=SeriesStr[0]=OptionSymbol[0]=0;

	// Get the DateStrike string, Series string and Underlying symbol string
	// NOTE: VERY IMPORTANT TO NOT PASS NULL POINTERS IN AS ADDY FOR A STRING!!
	// SOME OF THESE STRINGS MAY OCCASIONALLY BE NULL 
	if (pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsUnderlying)
	  strcpy(UnderlyingStr,pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsUnderlying->String);
	if (pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsSeriesChain)
	  strcpy(SeriesStr,pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsSeriesChain->String);
	
	// If pnxsDateAndStrike->String[1] == ' ', then this symbol is in new OSI format. 	
    if (pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[1]==' ')			
    {
	    // Construct OSI Symbol, take out 'o' from symbol root (String+1)	

		sprintf(OptionSymbol,"%-6s%02d%02d%02d%c%08d",
		        pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,
			    pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Year-2000,
				pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Month,
				pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Day,
				(pNxCoreMsg->coreHeader.pnxOptionHdr->PutCall == 0) ? 'C' : 'P',
				pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice);

		// Print the option symbol, underlying symbol and the series chain
		cout << "Symbol Spin for: " <<
				" NxSym: " 			<< pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
				" OSI Sym: " 		<< OptionSymbol <<
				" Strike: " 		<< fixed << setprecision(2) << StrikePrice <<
				" Underlying: " 	<< UnderlyingStr <<
				" Series Chain: " 	<< SeriesStr << endl;
    }

	// Otherwise the symbol is in old OPRA format.
	// Take out 'o' from symbol series (String+1)
	else
	{
	  sprintf(OptionSymbol,"%s%c%c",
		     pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,
		     pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[0],
	         pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[1]);	  

		// Print the option symbol, underlying symbol and the series chain
		cout << "Symbol Spin for: " <<
			" NxSym: " 				<< pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
			" OPRA Sym: " 			<< OptionSymbol <<
			" Strike: " 			<< fixed << setprecision(2) << StrikePrice <<
			" Underlying: " 		<< UnderlyingStr <<
			" Series Chain: " 		<< SeriesStr << endl;
	}
						
  }

  // Otherwise this is another equity type (stock, future, etc).
  else 
  {
	// Print the symbol and listed exchange for this spin message
	cout << "Symbol Spin for: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
			" Listed Exg: " << pNxCoreMsg->coreHeader.ListedExg << endl;
  }
}
