////////////////////////////////////////////////////
//// SampleApp3.cpp                             ////
//// Sample App for NxCore in C++               ////
//// Author: Jeffrey Donovan, Jeremy Truelove   ////
//// Date: 2017-02-09                           ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// Handling the NxCore Category Message       ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

#include "StdAfx.h"

#include <cstdlib>
#include <iostream>    				// cout, endl
#include <iomanip>					// setw, fixed, setprecision
#include "NxCoreAPI_Wrapper_C++.h"  // NxCore C++ wrapper with NxCoreClass

// Couple forwards - functions are below
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreCategory(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);

// Global Instance of NxCore Class
NxCoreClass NxCore;

// Main Entry Point for app.	
int main(int argc, char* argv[])
{
	int returnValue;
	if(argc < 2) {
		cout << "SampleApp03 dll-file [tape-name]" << endl;
		exit(EXIT_SUCCESS);
	}
    cout << "NxCore C++ SampleApp03 Start." << endl;

    // If we can load the NxCore libnx.so
	if (NxCore.LoadNxCore(argv[1]))
    {
	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. Use control flags
	  // to eliminate Level1 quotes, Level2 quotes and all OPRA
	  if (argv[1])
	  {
		cout << "Processing the tape: " << argv[2] << endl;
		returnValue = NxCore.ProcessTape(argv[2], NULL, (NxCF_EXCLUDE_QUOTES | NxCF_EXCLUDE_QUOTES2 | NxCF_EXCLUDE_OPRA), 0, (NxCoreCallback)OnNxCoreCallback);
	  }
	  else
		returnValue = NxCore.ProcessTape("", NULL, (NxCF_EXCLUDE_QUOTES | NxCF_EXCLUDE_QUOTES2 | NxCF_EXCLUDE_OPRA), 0, (NxCoreCallback)OnNxCoreCallback);

	  // output return value from processTape(), helps in troubleshooting potential issues
	  NxCore.ProcessReturnValue(returnValue);
                    
      // Unload NxCore DLL
	  NxCore.UnloadNxCore();
    }
	else {
	  cout << "Failed to load NxCore Library: " << dlerror() << endl;
	  exit(EXIT_FAILURE);
	}

	cout << "NxCore C++ SampleApp03 Stop." << endl;
	exit(EXIT_SUCCESS);
}


// The NXCore Callback Function	
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{ 
  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
	// NxCore Status Message
	case NxMSG_STATUS:
	     OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		 break;

    // NxCore Category Message
	case NxMSG_CATEGORY:
	     OnNxCoreCategory(pNxCoreSys, pNxCoreMsg);
	     break;
  }
	  
  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;  
}


// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
    // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
	  cout.fill('0');
	  cout << "NxCore Time: " <<
		setw(2) << (int) pNxCoreSys->nxDate.Month << "/" <<
		setw(2) << (int) pNxCoreSys->nxDate.Day << "/" <<
		setw(2) << (int) pNxCoreSys->nxDate.Year << " " <<

		setw(2) << (int) pNxCoreSys->nxTime.Hour << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Minute << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Second << endl;
	}
	
	// Print the specific NxCore status message
	switch (pNxCoreSys->Status) {
		case NxCORESTATUS_COMPLETE:
			cout << "NxCore Complete Message." << endl;
			break;

		case NxCORESTATUS_INITIALIZING:
			cout << "NxCore Initialize Message." << endl;
			cout << "libnx.so version is v" <<
				NxCore.GetMajorVersion(pNxCoreSys->DLLVersion) << "." <<
				NxCore.GetMinorVersion(pNxCoreSys->DLLVersion) << "." <<
				NxCore.GetBuildVersion(pNxCoreSys->DLLVersion) << endl;
			break;

		case NxCORESTATUS_SYNCHRONIZING:
			cout << "NxCore Synchronizing Message." << endl;
			break;

		case NxCORESTATUS_WAITFORCOREACCESS:
			cout << "NxCore Wait For Access." << endl;
			break;

		case NxCORESTATUS_RESTARTING_TAPE:
			cout << "NxCore Restart Tape Message." << endl;
			break;

		case NxCORESTATUS_ERROR:
			cout << "NxCore Error." << endl;
			break;

		case NxCORESTATUS_RUNNING:
			break;

	}
}

// OnNxCoreCategory: Function to handle NxCore Category messages.
// In this example we only capture category 4 and category 13 messages.
//--------------------------------------------------------------
void OnNxCoreCategory(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  // Category field pointer
  NxCategoryField* pField;
  	  	  
  // Do something based on the category num
  switch (pNxCoreMsg->coreData.Category.pnxStringCategory->Atom)
  {		 
		// Category 4, Equity Symbol Information
		case 4:

		  // Grab a pointer to the field
		  pField = &pNxCoreMsg->coreData.Category.pnxFields[0];

		  // Is field set?
		  if (pField->Set == 1)
		  {			

			// Print the Company Name			
			cout << "Category 4 for Symbol: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String << "," <<
					 " Company Name= " << pField->data.StringZ << endl;
		  }		  
		  break;


		// Category 13, 52 Week High/Low
		case 13:
		  double High52 = 0.0, Low52 = 0.0;
		  NxDate HighDate;
		  NxDate LowDate; 
		  memset(&HighDate,0,sizeof(NxDate));
		  memset(&LowDate,0,sizeof(NxDate));
		  
		  // Grab a pointer to the 52 week high date
		  pField = &pNxCoreMsg->coreData.Category.pnxFields[0];
		  if (pField->Set == 1)
			memcpy(&HighDate,&pField->data.nxDate,sizeof(NxDate));
		  
		  // Grab a pointer to the 52 week low date
		  pField = &pNxCoreMsg->coreData.Category.pnxFields[1];
		  if (pField->Set == 1)
			memcpy(&LowDate,&pField->data.nxDate,sizeof(NxDate));
		  
		  // Grab a pointer to the 52 week high field
		  pField = &pNxCoreMsg->coreData.Category.pnxFields[2];

		  // Get the 52 week high
		  if (pField->Set == 1)
			High52 = NxCore.PriceToDouble(pField->data.nxPrice.Price, pField->data.nxPrice.PriceType);

		  // Grab a pointer to the 52 week low field
		  pField = &pNxCoreMsg->coreData.Category.pnxFields[3];

		  // Get the 52 week low
		  if (pField->Set == 1)
			Low52 = NxCore.PriceToDouble(pField->data.nxPrice.Price, pField->data.nxPrice.PriceType);

		  cout.fill('0');
		  cout << "Category 13 for Symbol: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String << ", " <<
					  setw(2) << (int) HighDate.Month << "/" <<
					  setw(2) << (int) HighDate.Day << "/" <<
					  setw(4) << (int) HighDate.Year <<
				  " High52= " << fixed << setprecision(2) << High52 << ", " <<
					  setw(2) << (int) LowDate.Month << "/" <<
					  setw(2) << (int) LowDate.Day << "/" <<
					  setw(4) << (int) LowDate.Year <<
				  " Low52= " << fixed << setprecision(2) <<  Low52 << endl;
		  break;
	  }	  
	}

