////////////////////////////////////////////////////
//// SampleApp4.cpp                             ////
//// Sample App for NxCore in C++               ////
//// Author: Jeffrey Donovan, Jeremy Truelove   ////
//// Date: 2017-02-09                           ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// Handling NxCore Trade and Quote Messages   ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

#include "StdAfx.h"

#include <cstdlib>
#include <iostream>    				// cout, endl
#include <iomanip>					// setw, fixed, setprecision
#include "NxCoreAPI_Wrapper_C++.h"  // NxCore C++ wrapper with NxCoreClass

// Couple forwards - functions are below
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreExgQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreMMQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);

// Global Instance of NxCore Class
NxCoreClass NxCore;

// Main Entry Point for app.	
int main(int argc, char* argv[])
{
	int returnValue;

	if(argc < 2) {
		cout << "SampleApp04 dll-file [tape-name]" << endl;
		exit(EXIT_SUCCESS);
	}
    cout << "NxCore C++ SampleApp04 Start." << endl;

    // If we can load the NxCore libnx.so
	if (NxCore.LoadNxCore(argv[1]))
    {        
	  
	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  if (argv[2])
	  {
		cout << "Processing the tape: " << argv[2] << endl;
		returnValue = NxCore.ProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
	  }
	  else
		returnValue = NxCore.ProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);

	  // output return value from processTape(), helps in troubleshooting potential issues
	  NxCore.ProcessReturnValue(returnValue);

      // Unload NxCore DLL
	  NxCore.UnloadNxCore();
    }
    else {
	  cout << "Failed to load NxCore Library: " << dlerror() << endl;
	  exit(EXIT_FAILURE);
	}

	cout << "NxCore C++ SampleApp04 Stop." << endl;
	exit(EXIT_SUCCESS);
}


// The NXCore Callback Function	
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{ 
  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
	// NxCore Status Message
	case NxMSG_STATUS:
	     OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		 break;

    // NxCore Trade Message
	case NxMSG_TRADE:
		 OnNxCoreTrade(pNxCoreSys, pNxCoreMsg);
		 break;

	// NxCore Level1 Quote Message
	case NxMSG_EXGQUOTE:
		 OnNxCoreExgQuote(pNxCoreSys, pNxCoreMsg);
		 break;

	// NxCore Level2 Quote Message
	case NxMSG_MMQUOTE:
		 OnNxCoreMMQuote(pNxCoreSys, pNxCoreMsg);
		 break;
  }
	  
  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;  
}


// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
    // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
	  cout.fill('0');
	  cout << "NxCore Time: " <<
		setw(2) << (int) pNxCoreSys->nxDate.Month << "/" <<
		setw(2) << (int) pNxCoreSys->nxDate.Day << "/" <<
		setw(4) << (int) pNxCoreSys->nxDate.Year << " " <<

		setw(2) << (int) pNxCoreSys->nxTime.Hour << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Minute << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Second << endl;
	}
	
	// Print the specific NxCore status message
	switch (pNxCoreSys->Status) {
		case NxCORESTATUS_COMPLETE:
			cout << "NxCore Complete Message." << endl;
			break;

		case NxCORESTATUS_INITIALIZING:
			cout << "NxCore Initialize Message." << endl;
			cout << "libnx.so version is v" <<
				NxCore.GetMajorVersion(pNxCoreSys->DLLVersion) << "." <<
				NxCore.GetMinorVersion(pNxCoreSys->DLLVersion) << "." <<
				NxCore.GetBuildVersion(pNxCoreSys->DLLVersion) << endl;
			break;

		case NxCORESTATUS_SYNCHRONIZING:
			cout << "NxCore Synchronizing Message." << endl;
			break;

		case NxCORESTATUS_WAITFORCOREACCESS:
			cout << "NxCore Wait For Access." << endl;
			break;

		case NxCORESTATUS_RESTARTING_TAPE:
			cout << "NxCore Restart Tape Message." << endl;
			break;

		case NxCORESTATUS_ERROR:
			cout << "NxCore Error." << endl;
			break;

		case NxCORESTATUS_RUNNING:
			break;

	}
}


// OnNxCoreTrade: Function to handle NxCore Trade messages.	
//--------------------------------------------------------------
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
	// Assign a pointer to the Trade data
	NxCoreTrade* Trade = ( NxCoreTrade*) &pNxCoreMsg->coreData.Trade;

	// Get the price and net change
	double Price = NxCore.PriceToDouble(Trade->Price,Trade->PriceType);
	double NetChange = NxCore.PriceToDouble(Trade->NetChange, Trade->PriceType);

	// Write out Symbol, Time, Price, NetChg, Size, Reporting Exg
	cout.fill('0');
	cout << "Trade for Symbol: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String << "," <<
		" Time: " <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Hour << ":" <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Minute << ":" <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Second <<
		" Price: " 	<< fixed << setprecision(2) << Price <<
		" NetChg: " << fixed << setprecision(2) << NetChange <<
		" Size: " 	<< Trade->Size <<
		" Exchg: " 	<< pNxCoreMsg->coreHeader.ReportingExg << endl;
}

	
// OnNxCoreQuote: Function to handle NxCore ExgQuote messages.	
//--------------------------------------------------------------
void OnNxCoreExgQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  // Assign a pointer to the ExgQuote data
  NxCoreExgQuote* Quote = (NxCoreExgQuote*) &pNxCoreMsg->coreData.ExgQuote;

  // Get bid and ask price
  double Bid = NxCore.PriceToDouble(Quote->coreQuote.BidPrice, Quote->coreQuote.PriceType);
  double Ask = NxCore.PriceToDouble(Quote->coreQuote.AskPrice, Quote->coreQuote.PriceType);

  // Write out Symbol, Time, Bid, Ask, Bidsize, Asksize, Reporting Exg
  cout.fill('0');
  cout << "ExgQuote for Symbol: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String << "," <<
	" Time: " <<
		setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Hour << ":" <<
		setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Minute << ":" <<
		setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Second <<
	" Bid: " 		<< fixed << setprecision(2) << Bid <<
	" Ask: " 		<< fixed << setprecision(2) << Ask <<
	" BidSize: " 	<<  Quote->coreQuote.BidSize <<
	" AskSise: " 	<<  Quote->coreQuote.AskSize <<
	" Exchg: " 		<< pNxCoreMsg->coreHeader.ReportingExg << endl;
}

	
// OnNxCoreMMQuote: Function to handle NxCore MMQuote messages.	
//--------------------------------------------------------------
void OnNxCoreMMQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  // Assign a pointer to the MMQuote data
  NxCoreMMQuote* Quote = (NxCoreMMQuote*) &pNxCoreMsg->coreData.MMQuote;
	  	  	  
  // Get bid and ask price
  double Bid = NxCore.PriceToDouble(Quote->coreQuote.BidPrice, Quote->coreQuote.PriceType);
  double Ask = NxCore.PriceToDouble(Quote->coreQuote.AskPrice, Quote->coreQuote.PriceType);

  // Write out Symbol, MarketMaker, Time, Bid, Ask, Bidsize, Asksize, Reporting Exg
  cout.fill('0');
  cout << "MMQuote for Symbol: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String << "," <<
	" MarketMaker: " << Quote->pnxStringMarketMaker->String <<
	" Time: " <<
		setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Hour << ":" <<
		setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Minute << ":" <<
		setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Second <<
	" Bid: " 		<< fixed << setprecision(2) << Bid <<
	" Ask: " 		<< fixed << setprecision(2) << Ask <<
	" BidSize: " 	<< Quote->coreQuote.BidSize <<
	" AskSise: " 	<< Quote->coreQuote.AskSize <<
	" Exchg: " 		<< pNxCoreMsg->coreHeader.ReportingExg << endl;
}
