////////////////////////////////////////////////////
//// SampleApp6.cpp                             ////
//// Sample App for NxCore in C++               ////
//// Author: Jeffrey Donovan, Jeremy Truelove   ////
//// Date: 2017-02-09                           ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// Retriving StateTrades, StateExgQuotes      ////
//// and StateMMQuotes                          ////
////                                            ////
//// NOTE: In this sample, we retrieve State    ////
////       items when NxCore delivers real time ////
////       Trade, Quote and MMQuote messages.   ////
////       This is NOT the most effective way   ////
////       to get the streaming data (see       ////
////       Sample4). However it does effectively////
////       demonstrate retrieving the State for ////
////       any symbol.                          ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

#include "StdAfx.h"

#include <cstdlib>
#include <iostream>    				// cout, endl
#include <iomanip>					// setw, fixed, setprecision
#include "NxCoreAPI_Wrapper_C++.h"  // NxCore C++ wrapper with NxCoreClass

// Couple forwards - functions are below
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnGetMMStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnGetExgQuoteStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnGetLastTradeStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);

// Global Instance of NxCore Class
NxCoreClass NxCore;

// Main Entry Point for app.	
int main(int argc, char* argv[])
{
	int returnValue;
	if(argc < 2) {
		cout << "SampleApp06 dll-file [tape-name]" << endl;
		exit(EXIT_SUCCESS);
	}
    cout << "NxCore C++ SampleApp06 Start." << endl;

    // If we can load the NxCore libnx.so
	if (NxCore.LoadNxCore(argv[1]))
    {        
	  
	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  if (argv[2])
	  {
		cout << "Processing the tape: " << argv[1] << endl;
		returnValue = NxCore.ProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
	  }
	  else
		  returnValue = NxCore.ProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);

	  // output return value from processTape(), helps in troubleshooting potential issues
	  NxCore.ProcessReturnValue(returnValue);

      // Unload NxCore DLL
	  NxCore.UnloadNxCore();
    }
    else {
	  cout << "Failed to load NxCore Library: " << dlerror() << endl;
	  exit(EXIT_FAILURE);
	}

	cout << "NxCore C++ SampleApp6 Stop." << endl;
	exit(EXIT_SUCCESS);
}


// The NXCore Callback Function	
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{ 
  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
	// NxCore Status Message
	case NxMSG_STATUS:
	     OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		 break;

    // NxCore Trade Message
	case NxMSG_TRADE:	  
		 OnGetLastTradeStates(pNxCoreSys, pNxCoreMsg);
		 break;

	// NxCore Level1 Quote Message
	case NxMSG_EXGQUOTE:	  
		 OnGetExgQuoteStates(pNxCoreSys, pNxCoreMsg);
		 break;

	// NxCore Level2 Quote Message
	case NxMSG_MMQUOTE:
		 OnGetMMStates(pNxCoreSys, pNxCoreMsg);
		 break;
  }
	  
  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;  
}


// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
    // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
		cout.fill('0');
		cout << "NxCore Time: " <<
			setw(2) << (int) pNxCoreSys->nxDate.Month << "/" <<
			setw(2) << (int) pNxCoreSys->nxDate.Day << "/" <<
			setw(2) << (int) pNxCoreSys->nxDate.Year << " " <<

			setw(2) << (int) pNxCoreSys->nxTime.Hour << ":" <<
			setw(2) << (int) pNxCoreSys->nxTime.Minute << ":" <<
			setw(2) << (int) pNxCoreSys->nxTime.Second << endl;
	}
	
	// Print the specific NxCore status message
    switch( pNxCoreSys->Status ) 
    {
	  case NxCORESTATUS_COMPLETE:
		   cout << "NxCore Complete Message." << endl;
		   break;

      case NxCORESTATUS_INITIALIZING:
		   cout << "NxCore Initialize Message." << endl;
		   break;

	  case NxCORESTATUS_SYNCHRONIZING:
		   cout << "NxCore Synchronizing Message." << endl;
		   break;

	  case NxCORESTATUS_WAITFORCOREACCESS:
		   cout << "NxCore Wait For Access." << endl;
		   break;

	  case NxCORESTATUS_RESTARTING_TAPE:
		   cout << "NxCore Restart Tape Message." << endl;
		   break;

      case NxCORESTATUS_ERROR:
		   cout << "NxCore Error." << endl;
		   break;
 	  
      case NxCORESTATUS_RUNNING:		   
		   break;

    }
}


// OnGetMMStates: Function to retrieve the current Market
// maker list for the issue, populated to the current time.
//----------------------------------------------------------
void OnGetMMStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  NxCoreStateMMQuotes pStatMMQuotes;

  // Retrieve MM Quote....make sure a valid return from function
  if (NxCore.StateGetMMQuotes(pNxCoreMsg->coreHeader.ReportingExg,
			                &pStatMMQuotes,
					        pNxCoreMsg->coreHeader.pnxStringSymbol)==0)
  {

    cout << "StateMMQuotes for " << pNxCoreMsg->coreHeader.pnxStringSymbol->String << ","
    		" Count=" << pStatMMQuotes.StateQuoteCount << endl;

    // Cycle through all quotes in the state
    for (int loop = 0; loop < pStatMMQuotes.StateQuoteCount; loop++)
    {
      // Get bid and ask price
	  double Bid = NxCore.PriceToDouble(pStatMMQuotes.StateMMQuotes[loop].BidPrice, pStatMMQuotes.PriceType);
	  double Ask = NxCore.PriceToDouble(pStatMMQuotes.StateMMQuotes[loop].AskPrice, pStatMMQuotes.PriceType);
	  int BidSize = pStatMMQuotes.StateMMQuotes[loop].BidSize;
	  int AskSize = pStatMMQuotes.StateMMQuotes[loop].AskSize;

	  cout << "Market Maker: " 	<< pStatMMQuotes.StateMMQuotes[loop].pnxStringMarketMaker->String <<
			  " Bid: " 			<< fixed << setprecision(2) << Bid <<
			  " Ask: " 			<< fixed << setprecision(2) << Ask <<
			  " BidSize: " 		<< BidSize <<
			  " AskSize: " 		<< AskSize << endl;
	}
  }  
}


// OnGetExgQuoteStates: Function to retrieve the current Exg quote
// list for the issue, populated to the current time.
//----------------------------------------------------------
void OnGetExgQuoteStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  NxCoreStateExgQuotes pStatExgQuotes;
  NxString *RequestString=NULL;

  // If an option....
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&
	  (pNxCoreMsg->coreHeader.pnxOptionHdr))
	  RequestString=pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike;

  // otherwise a non-option
  else
	  RequestString=pNxCoreMsg->coreHeader.pnxStringSymbol;

  // Retrieve Exg Quote....make sure a valid return from function
  if (NxCore.StateGetExgQuotes(&pStatExgQuotes,RequestString)==0)
  {

    cout << "StateExgQuotes for " << pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
    		" BestBidExg: " << pStatExgQuotes.BestBidExg <<
			" BestAskExg: " << pStatExgQuotes.BestAskExg << endl;

    // Cycle through all quotes in the state
    for (int loop = 0; loop < pStatExgQuotes.StateQuoteCount; loop++)
    {
      // Get bid and ask price
      double Bid = NxCore.PriceToDouble(pStatExgQuotes.StateExgQuotes[loop].BidPrice, pStatExgQuotes.PriceType);
      double Ask = NxCore.PriceToDouble(pStatExgQuotes.StateExgQuotes[loop].AskPrice, pStatExgQuotes.PriceType);
      int BidSize = pStatExgQuotes.StateExgQuotes[loop].BidSize;
      int AskSize = pStatExgQuotes.StateExgQuotes[loop].AskSize;
      unsigned short ReportingExg = pStatExgQuotes.StateExgQuotes[loop].ReportingExg;
      unsigned char Condition = pStatExgQuotes.StateExgQuotes[loop].QuoteCondition;

      cout << "Exg: " << ReportingExg <<
			  " Bid: " 			<< fixed << setprecision(2) << Bid <<
			  " Ask: " 			<< fixed << setprecision(2) << Ask <<
			  " BidSize: " 		<< BidSize <<
			  " AskSize: " 		<< AskSize <<
    		  " Condition: "  	<< (int) Condition << endl;
    }
  }
}


// OnGetLastTradeStates: Function to retrieve the current composite trade
// info for the issue, populated to the current time.
//----------------------------------------------------------
void OnGetLastTradeStates(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  NxCoreStateTrade pStatLastTrade;
  NxString *RequestString=NULL;

  // If an option....
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&
	  (pNxCoreMsg->coreHeader.pnxOptionHdr))
	  RequestString=pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike;

  // otherwise a non-option
  else
	  RequestString=pNxCoreMsg->coreHeader.pnxStringSymbol;

  // Retrieve Last Trade State....make sure a valid return from function
  if (NxCore.StateGetLastTrade(&pStatLastTrade,RequestString)==0)
  {
    cout << "StateLastTrade for " << pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
    		" Open: " 		<< fixed << setprecision(2) << NxCore.PriceToDouble(pStatLastTrade.Open, pStatLastTrade.PriceType) <<
			" High: " 		<< fixed << setprecision(2) << NxCore.PriceToDouble(pStatLastTrade.High, pStatLastTrade.PriceType) <<
			" Low: " 		<< fixed << setprecision(2) << NxCore.PriceToDouble(pStatLastTrade.Low, pStatLastTrade.PriceType) <<
			" Last: " 		<< fixed << setprecision(2) << NxCore.PriceToDouble(pStatLastTrade.Last, pStatLastTrade.PriceType) <<
			" Price: " 		<< fixed << setprecision(2) << NxCore.PriceToDouble(pStatLastTrade.Price, pStatLastTrade.PriceType) <<
			" NetChg: " 		<< fixed << setprecision(2) << NxCore.PriceToDouble(pStatLastTrade.NetChange, pStatLastTrade.PriceType) <<
			" TotalVol: " 	<< pStatLastTrade.TotalVolume << endl;
  }
}
