////////////////////////////////////////////////////
//// SampleApp7.cpp                             ////
//// Sample App for NxCore in C++               ////
//// Author: Jeffrey Donovan                    ////
//// Date: 2017-02-09                           ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// 1) User defined symbol spin function       ////
//// 2) Available NxCore Data file listings     ////
//// 3) Available NxCore DLL file listings      ////
//// 4) Retrieving strings from NxCore Tables   ////
//// 5) Get Date from N days                    ////
//// 6) Get N days from date                    ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

#include "StdAfx.h"

#include <cstdlib>
#include <iostream>    				// cout, endl
#include <iomanip>					// setw, fixed, setprecision
#include "NxCoreAPI_Wrapper_C++.h"  // NxCore C++ wrapper with NxCoreClass

// Couple forwards - functions are below
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
int OnUserSymbolSpin(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);

void GetDefinedString();
void GetTapeName();
void DateFromDays();
void DaysFromDate();

// Global Instance of NxCore Class
NxCoreClass NxCore;

// Main Entry Point for app.	
int main(int argc, char* argv[])
{
	int returnValue;
	if(argc < 2) {
		cout << "SampleApp07 dll-file [tape-name]" << endl;
		exit(EXIT_SUCCESS);
	}
    cout << "NxCore C++ SampleApp07 Start." << endl;

    // If we can load the NxCore libnx.so
	if (NxCore.LoadNxCore(argv[1]))
    {        
	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  if (argv[2]){
		cout << "Processing the tape: " << argv[2] << endl;
	    returnValue = NxCore.ProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
	  }
	  else
		returnValue = NxCore.ProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
                    
	  // output return value from processTape(), helps in troubleshooting potential issues
	  NxCore.ProcessReturnValue(returnValue);
      // Unload NxCore DLL
	  NxCore.UnloadNxCore();
    }
    else {
	  cout << "Failed to load NxCore Library: " << dlerror() << endl;
	  exit(EXIT_FAILURE);
	}

	cout << "NxCore C++ SampleApp07 Stop." << endl;
	exit(EXIT_SUCCESS);
}


// The NXCore Callback Function	
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{ 
  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
	// NxCore Status Message
	case NxMSG_STATUS:
	     OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		 break;
  }
	  
  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;  
}


// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
    // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
		cout.fill('0');
		cout << "NxCore Time: " <<
			setw(2) << (int) pNxCoreSys->nxDate.Month << "/" <<
			setw(2) << (int) pNxCoreSys->nxDate.Day << "/" <<
			setw(2) << (int) pNxCoreSys->nxDate.Year << " " <<

			setw(2) << (int) pNxCoreSys->nxTime.Hour << ":" <<
			setw(2) << (int) pNxCoreSys->nxTime.Minute << ":" <<
			setw(2) << (int) pNxCoreSys->nxTime.Second << endl;
	}
	
	// Print the specific NxCore status message
	switch (pNxCoreSys->Status) {
		// Capture a SYMBOL SPIN Message
		case NxCORESTATUS_SYMBOLSPIN:
			switch (pNxCoreSys->StatusData) {

			// If initial symbol spin is complete, test the functions in this
			// sample application
			case NxCSSYMBOLSPIN_COMPLETE:

				// User defined symbol spin
				cout << "NxCore Initial Spin Complete Message." << endl;
				cout << "Press <Enter> to run a user define spin." << endl;
				cin.get();

				NxCore.SpinSymbols(NxSPIN_ALL_EXGS, 0, 1, (NxCoreCallback)OnUserSymbolSpin, 0);

				// Get defined string from NxCore tables
				cout << "UserDefine Spin Complete!" << endl;
				cout << "Press <Enter> to get defined strings for exchanges." << endl;
				cin.get();

				GetDefinedString();

				// Get current tape filename
				cout << "Press <Enter> to get the Tape Name." << endl;
				cin.get();

				GetTapeName();

				// Get date from N days
				cout << "Press <Enter> to get a date from N days." << endl;
				cin.get();

				DateFromDays();

				// Get N Days from date
				cout << "Press <Enter> to get NDays from a date." << endl;
				cin.get();

				DaysFromDate();

				// Finish!
				cout << "Press <Enter> to continue with tape." << endl;
				cin.get();
				break;
			}
			break;

		case NxCORESTATUS_COMPLETE:
			cout << "NxCore Complete Message." << endl;
			break;

		case NxCORESTATUS_INITIALIZING:
			cout << "NxCore Initialize Message." << endl;
			cout << "libnx.so version is v"
					<< NxCore.GetMajorVersion(pNxCoreSys->DLLVersion) << "."
					<< NxCore.GetMinorVersion(pNxCoreSys->DLLVersion) << "."
					<< NxCore.GetBuildVersion(pNxCoreSys->DLLVersion) << endl;
			break;

		case NxCORESTATUS_SYNCHRONIZING:
			cout << "NxCore Synchronizing Message." << endl;
			break;

		case NxCORESTATUS_WAITFORCOREACCESS:
			cout << "NxCore Wait For Access." << endl;
			break;

		case NxCORESTATUS_RESTARTING_TAPE:
			cout << "NxCore Restart Tape Message." << endl;
			break;

		case NxCORESTATUS_ERROR:
			cout << "NxCore Error." << endl;
			break;

		case NxCORESTATUS_RUNNING:
			break;

	}
}


// GetDefinedString: Get string description from NxCore table definitions.
//------------------------------------------------------------------------
void GetDefinedString()
{
  // Loop for 60 exchanges...
  for (int loop = 1; loop < 68; loop++)
  {
	char Exg[260];

	// Get the defined name for this exchange
	strcpy(Exg,NxCore.GetDefinedString((int)NxST_EXCHANGE, loop));		

	// Print exchange num and name
	cout << "Exg #" << loop <<
			" = " 	<< Exg << endl;
  }
}


// GetTapeName: Get filename for current tape
//-------------------------------------------
void GetTapeName()
{
  char TapeName[260];
	  
  // Get current tape name, convert to string type
  NxCore.GetTapeName(TapeName,254);
  
  // Print tape name
  cout << "TapeName " << TapeName << endl;
}


// DateFromDays: Get date from N Days
//-----------------------------------
void DateFromDays()
{
  NxDate Date;
	  
  // Set NDays to get date for
  Date.NDays=32000;

  // Call DateFromNDays function
  NxCore.DateFromNDays(&Date);

  // print NDays and date
  cout.fill('0');
  cout << "Date for " << Date.NDays << " Days: " <<
		  setw(2) << (int) Date.Month << "/" <<
		  setw(2) << (int) Date.Day << "/" <<
		  setw(4) << (int) Date.Year << endl;
}


// DaysFromDate: Get N days from date
//-----------------------------------
void DaysFromDate()
{
  NxDate Date;

  // Set date to get NDays from
  Date.Month=1;
  Date.Day  =1;
  Date.Year = 2010;

  // Call DateFromYMD function
  NxCore.DateFromYMD(&Date);

  // Print date and NDays
  cout.fill('0');
  cout << "Date " <<
	  setw(2) << (int) Date.Month << "/" <<
	  setw(2) << (int) Date.Day << "/" <<
	  setw(4) << (int) Date.Year <<
	  " Days: " << Date.NDays << endl;
}


// OnUserListTapes: Callback with abailable tape filenames on sytstem
//-------------------------------------------------------------------
int OnUserListTapes(void *pUser, const NxCoreTapeFile *pTapeFile)
{
  // Print the tape filename
  cout << "Filename: " << pTapeFile->PathnameStrZ << endl;
	  
  // Continue to next
  return NxCALLBACKRETURN_CONTINUE;
}


// OnUserListAPIDLLs: Callback with abailable NxCore DLL's on sytstem
//-------------------------------------------------------------------
int OnUserListAPIDLLs(void * pUser, const NxCoreAPIDLLFile *pDLLFile)
{
  // Print the DLL filename
  cout << "Filename: " << pDLLFile->PathnameStrZ << endl;

  // Contiue to next
  return NxCALLBACKRETURN_CONTINUE;
}


// OnUserSymbolSpin:  User defined symbol spin callback
//------------------------------------------------------
int OnUserSymbolSpin(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{

  // Print the spin message sym, exg, time, date and session ID	  
  cout.fill('0');
  cout << "UserSymbol Spin for: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
		" Listed Exg: " << pNxCoreMsg->coreHeader.ListedExg <<
		" Time: " <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Hour << ":" <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Minute << ":" <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Second << " " <<

			setw(2) << (int) pNxCoreMsg->coreHeader.nxSessionDate.Month << "/" <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxSessionDate.Day << "/" <<
			setw(4) << (int) pNxCoreMsg->coreHeader.nxSessionDate.Year <<
			pNxCoreMsg->coreHeader.SessionID << endl;

  // Continue to next
  return NxCALLBACKRETURN_CONTINUE;
}
