//////////////////////////////////////////////////////
//// SampleApp8.cpp                               ////
//// Sample App for NxCore in C++                 ////
//// Author: Jeffrey Donovan, Jeremy Truelove     ////
//// Date: 2017-02-09                             ////
//////////////////////////////////////////////////////
//// Demonstrates:                                ////
//// 1) User defined symbol spin function         ////
//// 2) Setting UserData1 and UserData2 values    ////
//// 3) Retrieving UserData1 and UserData2 values ////
//// 4) Determining if a tape is OSI from the tape////
////    version number.                           ////
//// 5) Constructing Option Symbols in OSI or     ////
////    old OPRA format                           ////
//////////////////////////////////////////////////////
//// To Read as Written:                          ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs       ////
//////////////////////////////////////////////////////

#include "StdAfx.h"

#include <cstdlib>
#include <iostream>    				// cout, endl
#include <iomanip>					// setw, fixed, setprecision
#include <cstdlib>					// atoi
#include <stdio.h>					// sprintf
#include "NxCoreAPI_Wrapper_C++.h"  // NxCore C++ wrapper with NxCoreClass

// Couple forwards - functions are below
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreSymbolSpin(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
int OnUserSymbolSpin_NonOptions(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
int OnUserSymbolSpin_OptionsTraditional(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
int OnUserSymbolSpin_OptionsOSI(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);

// Global Instance of NxCore Class
NxCoreClass NxCore;

// We'll use these to set the UserData1 and UserData1 members of NxString in the
// initial symbol spin, and retrieve the values from two UserDefined Spins
int MainCounter = 0;
int MainOptionCounter = 0;

unsigned char IsOSI=0;

// Main Entry Point for app.	
int main(int argc, char* argv[])
{
	int returnValue;
	if(argc < 2) {
		cout << "SampleApp08 dll-file [tape-name]" << endl;
		exit(EXIT_SUCCESS);
	}
    cout << "NxCore C++ SampleApp08 Start." << endl;

	if (NxCore.LoadNxCore(argv[1]))
	{
	  
	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  if (argv[2]){
		cout << "Processing the tape: " << argv[2] << endl;
	    returnValue = NxCore.ProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
	  }
	  else
		returnValue = NxCore.ProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
                    
	  // output return value from processTape(), helps in troubleshooting potential issues
	  NxCore.ProcessReturnValue(returnValue);

      // Unload NxCore DLL
	  NxCore.UnloadNxCore();
    }
    else {
	  cout << "Failed to load NxCore Library: " << dlerror() << endl;
	  exit(EXIT_FAILURE);
	}

	cout << "NxCore C++ SampleApp08 Stop." << endl;
	exit(EXIT_SUCCESS);
}

// The NXCore Callback Function	
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{ 

  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
	// NxCore Symbol Spin message
	case NxMSG_SYMBOLSPIN:
	     OnNxCoreSymbolSpin(pNxCoreSys, pNxCoreMsg);
		 break;

	// NxCore Status Message
	case NxMSG_STATUS:
	     OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		 break;
  }
	  
  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;  
}


// ExtractTapeVersion  - Extract Tape Version Number
//----------------------------------------------------------
void ExtractTapeVersion(char *TapeString,
					    char *VersionString,
					    int *Major,int *Minor,int *Sub)
{
  if ((!TapeString)||(!VersionString)) return;

  char *pt=&TapeString[12]; // Version num starts at char 13 in TapeString
  char *pv=VersionString;
  char extracted[3];  
  char *pe=extracted;
  unsigned char vpos=0;
  
  // Extract version string and numeric components   
  while ((*pt)&&(*pt!='\n'))
  {
	// '.' seperates the major/minor/sub version nums
	if (*pt=='.')
	{
	  // Extract Major and Minor versions
	  *pe=0;	  
	  if (vpos==0) *Major=atoi(extracted);			    
	  else	       *Minor=atoi(extracted);	
	  				 	
	  ++vpos;
	  *pv++ = *pt++;	  

	  pe=extracted;
	}
	else	
	  *pe++ = *pv++ = *pt++;		
  } 
  // Set version string end to null
  *pv=0;

  // Extract final Sub version
  *pe=0;
  *Sub=atoi(extracted);
}

// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
    // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
	  cout.fill('0');
	  cout << "NxCore Time: " <<
		setw(2) << (int) pNxCoreSys->nxDate.Month << "/" <<
		setw(2) << (int) pNxCoreSys->nxDate.Day << "/" <<
		setw(4) << (int) pNxCoreSys->nxDate.Year << " " <<

		setw(2) << (int) pNxCoreSys->nxTime.Hour << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Minute << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Second << endl;
	}
	
	// Print the specific NxCore status message
    switch( pNxCoreSys->Status ) 
    {
	  // Capture a SYMBOL SPIN Message
		case NxCORESTATUS_SYMBOLSPIN:
		     switch (pNxCoreSys->StatusData)
		     {

			   // If initial symbol spin is complete, test the functions in this
			   // sample application 
			   case NxCSSYMBOLSPIN_COMPLETE:

			     // User defined symbol spin
			     cout << "NxCore Initial Spin Complete Message." << endl;
			     cout << "Press <Enter> to run a user define spin for non-options." << endl;
			     cin.get();

			     NxCore.SpinSymbols(NxSPIN_ALL_EXGS, 0, 1, (NxCoreCallback)OnUserSymbolSpin_NonOptions, 0);

				 // User defined symbol spin
			     cout << "NxCore User Spin Complete Message." << endl;
			     cout << "Press <Enter> to run a user define spin for options." << endl;
			     cin.get();

				 // If the tape is an OSI tape, point the spin to the OSI handler,
				 // otherwise point the spin to the Traditional OPRA handler
				 if (IsOSI)
			       NxCore.SpinSymbols(NxSPIN_ALL_EXGS, 1, 1, (NxCoreCallback)OnUserSymbolSpin_OptionsOSI, 0);
				 else
				   NxCore.SpinSymbols(NxSPIN_ALL_EXGS, 1, 1, (NxCoreCallback)OnUserSymbolSpin_OptionsTraditional, 0);

			     // Finish!			     
			     cout << "Press <Enter> to continue with tape." << endl;
			     cin.get();
			     break;
		   }
		   break;

	  case NxCORESTATUS_COMPLETE:
		   cout << "NxCore Complete Message." << endl;
		   break;

      case NxCORESTATUS_INITIALIZING:
		  cout << "NxCore Initialize Message." << endl;
		  cout << "libnx.so version is v"
				<< NxCore.GetMajorVersion(pNxCoreSys->DLLVersion) << "."
				<< NxCore.GetMinorVersion(pNxCoreSys->DLLVersion) << "."
				<< NxCore.GetBuildVersion(pNxCoreSys->DLLVersion) << endl;
		  break;

	  case NxCORESTATUS_SYNCHRONIZING:
		   cout << "NxCore Synchronizing Message." << endl;
		   break;

	  case NxCORESTATUS_WAITFORCOREACCESS:
		   cout << "NxCore Wait For Access." << endl;
		   break;

	  case NxCORESTATUS_RESTARTING_TAPE:
		   cout << "NxCore Restart Tape Message." << endl;
		   break;

      case NxCORESTATUS_ERROR:
		   cout << "NxCore Error." << endl;
		   break;
 	  
      case NxCORESTATUS_RUNNING:		   
		   break;

	  // Tape Open Message. Get the tape version number
	  case NxCORESTATUS_TAPEOPEN:		   
           char VString[12];
           int Major = 0, Minor = 0, Sub = 0;

		   // Extract tape version from TapeDisplay string
		   ExtractTapeVersion((char *)pNxCoreSys->TapeDisplay,VString,&Major,&Minor,&Sub);
		   cout.fill('0');
		   cout << "NxCore Tape Open Message." << endl;
		   cout << "Tape Version String: " << VString <<
				   " Tape Version Numeric: v" << Major << "." <<
				   Minor << "." <<
				   Sub <<
				   " System Date: " <<
				   setw(2) << (int) pNxCoreSys->nxDate.Month << "/" <<
				   setw(2) << (int) pNxCoreSys->nxDate.Day << "/" <<
				   setw(4) << (int) pNxCoreSys->nxDate.Year << endl;

		   // If tape version is greater than 1.20.x then this is an OSI tape
	   	   if ((Major>1)||((Major==1)&&(Minor>20))) IsOSI=1;

		   cout << "Press <Enter> to Continue..." << endl;
		   cin.get();
		   break;
    }
}


// OnNxCoreSymbolSpin: Function to handle NxCore Symbol Spin messages
//-------------------------------------------------------------------
void OnNxCoreSymbolSpin(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  int UData1 = 0;
  int UData2 = 0;

  // If this is an option, and there is a valid option header, use it
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&(pNxCoreMsg->coreHeader.pnxOptionHdr != NULL))
  {   		
	char OptionSymbol[40];

	// SET UserData1 for the option to MainOptionCounter (increment the counter as we do).
	UData1=pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData1 = ++MainOptionCounter;

	// SET UserData2 to some odd value based on Nx Clock
	UData2=pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData2 = (pNxCoreSys->nxTime.Second * pNxCoreSys->nxTime.Millisecond);


	// If tape is determined to be an OSI tape...
    if (IsOSI)		
    {
	    // Construct OSI symbol, take out 'o' from symbol root (String+1)
		sprintf(OptionSymbol,"%-6s%02d%02d%02d%c%08d",
		        pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,					 
			    pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Year-2000,
				pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Month,
				pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Day,
				(pNxCoreMsg->coreHeader.pnxOptionHdr->PutCall == 0) ? 'C' : 'P',
				pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice);

		// Print the option symbol and user data
		cout << "Initial Symbol Spin for: NxSym: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
				" OSI Sym: " << OptionSymbol <<
				" UserData1: " << UData1 <<
				" UserData2: " <<  UData2 << endl;
    }
	// Otherwise the symbol is in old OPRA format.
	// Take out 'o' from symbol series (String+1)
	else
	{
	  sprintf(OptionSymbol,"%s%c%c",
		     pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,
		     pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[0],
	         pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[1]);	  

	  // Print the option symbol and user data
	  cout << "Initial Symbol Spin for: NxSym: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
			  " OPRA Sym: " << OptionSymbol <<
			  " UserData1: " << UData1 <<
			  " UserData2: " << UData2 << endl;
	}
  }

  // Otherwise this is another equity type (stock, future, etc).
  else 
  {
	// SET UserData1 to the MainCounter (increment the counter as we do).
	UData1=pNxCoreMsg->coreHeader.pnxStringSymbol->UserData1 = ++MainCounter;

	// SET UserData2 to some odd value based on Nx Clock
	UData2 = pNxCoreMsg->coreHeader.pnxStringSymbol->UserData2 = (pNxCoreSys->nxTime.Second * pNxCoreSys->nxTime.Millisecond);

	// Print the symbol and listed exchange for this spin message
	cout << "Initial Symbol Spin for: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
			" Listed Exg: " << pNxCoreMsg->coreHeader.ListedExg <<
			" UserData1=" << UData1 <<
			" UserData2=" << UData2 << endl;
  }
}


// OnUserSymbolSpin:  User defined symbol spin callback for non options
//---------------------------------------------------------------------
int OnUserSymbolSpin_NonOptions(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  int UData1 = 0;
  int UData2 = 0;

  // Retrieve the UseerData1 and UserData2 values set in the initial spin
  UData1 = pNxCoreMsg->coreHeader.pnxStringSymbol->UserData1;
  UData2 = pNxCoreMsg->coreHeader.pnxStringSymbol->UserData2;

  // Print the symbol and listed exchange for this spin message
  cout << "User Symbol Spin for: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
	  " Listed Exg: " << pNxCoreMsg->coreHeader.ListedExg <<
	  " UserData1=" << UData1 <<
	  " UserData2=" << UData2 << endl;
	  
  // Continue to next
  return NxCALLBACKRETURN_CONTINUE;
}


// OnUserSymbolSpin:  User defined symbol spin callback for tape with traditional OPRA option symbols
//------------------------------------------------------
int OnUserSymbolSpin_OptionsTraditional(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  int UData1 = 0;
  int UData2 = 0;

  
  // If this is an option, and there is a valid option header, use it
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&(pNxCoreMsg->coreHeader.pnxOptionHdr != NULL))
  {    
	char OptionSymbol[25];

	// Retrieve the UseerData1 and UserData2 values set in the initial spin
	UData1 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData1;
	UData2 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData2;
								
    sprintf(OptionSymbol,"%s%c%c",
		     pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,
		     pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[0],
	         pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->String[1]);	  

    // Print the option symbol and user data
    cout << "UserSymbol Spin for: NxSym: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
    		" OPRA Sym: " << OptionSymbol <<
    		" UserData1: " << UData1 <<
			" UserData2: " << UData2 << endl;
  }
	  
  // Continue to next
  return NxCALLBACKRETURN_CONTINUE;
}

// OnUserSymbolSpin:  User defined symbol spin callback for tape with OSI option symbols
//------------------------------------------------------
int OnUserSymbolSpin_OptionsOSI(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
  int UData1 = 0;
  int UData2 = 0;

  
  // If this is an option, and there is a valid option header, use it
  if ((pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='o')&&(pNxCoreMsg->coreHeader.pnxOptionHdr != NULL))
  {    
	char OptionSymbol[40];

	// Retrieve the UseerData1 and UserData2 values set in the initial spin
	UData1 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData1;
	UData2 = pNxCoreMsg->coreHeader.pnxOptionHdr->pnxsDateAndStrike->UserData2;
								
    // Construct OSI Symbol, take out 'o' from symbol root (String+1)	
	sprintf(OptionSymbol,"%-6s%02d%02d%02d%c%08d",
	        pNxCoreMsg->coreHeader.pnxStringSymbol->String+1,					 
		    pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Year-2000,
			pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Month,
			pNxCoreMsg->coreHeader.pnxOptionHdr->nxExpirationDate.Day,
			(pNxCoreMsg->coreHeader.pnxOptionHdr->PutCall == 0) ? 'C' : 'P',
			pNxCoreMsg->coreHeader.pnxOptionHdr->strikePrice);

	// Print the option symbol and user data
	cout << "UserSymbol Spin for: NxSym: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String <<
			" OSI Sym: " << OptionSymbol <<
			" UserData1: " << UData1 <<
			" UserData2: " << UData2 << endl;
  }
	  
  // Continue to next
  return NxCALLBACKRETURN_CONTINUE;
}
