//////////////////////////////////////////////////////
//// SampleApp10.cpp                              ////
//// Sample App for NxCore in C++                 ////
//// Author: Jeffrey Donovan, Jeremy Truelove     ////
//// Date: 2017-02-09                             ////
//////////////////////////////////////////////////////
//// Demonstrates:                                ////
//// 1) Saving to a NxCore State Tape             ////
//// 2) Running from an NxCore State Tape         ////
//////////////////////////////////////////////////////
//// To Read as Written:                          ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs       ////
//////////////////////////////////////////////////////

#include "StdAfx.h"

#include <cstdlib>
#include <iostream>    				// cout, endl
#include <iomanip>					// setw, fixed, setprecision
#include "NxCoreAPI_Wrapper_C++.h"  // NxCore C++ wrapper with NxCoreClass

// Couple forwards - functions are below
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);

// Global Instance of NxCore Class
NxCoreClass NxCore;

unsigned char SavedState=0;
char StateFileFilename[255];

// Main Entry Point for app.	
int main(int argc, char* argv[])
{
	int returnValue;

	if(argc < 2) {
		cout << "SampleApp10 dll-file [tape-name]" << endl;
		exit(EXIT_SUCCESS);
	}
	cout << "NxCore C++ SampleApp10 Start." << endl;

	strcpy(StateFileFilename,"STATESAVE.TMP");

	if (NxCore.LoadNxCore(argv[1]))
    {        
	  cout << "The application will process trades until 7:00 AM and then save the state," << endl <<
	  "exit and reload from the state tape." << endl <<
	  "Press <ENTER> to continue." << endl;
	  cin.get();
	  
	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  if (argv[2]){
        cout << "Processing the tape: " << argv[2] << endl;
	    returnValue = NxCore.ProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
	  }
	  else
		returnValue = NxCore.ProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);

	  // The current process has exited and the state tape has been saved!
	  cout << "Tape state saved and process exited." << endl <<
	  "Press <ENTER> to Load the state and continue." << endl;
	  cin.get();

	  // Reset this flag so we dont stop in the next call to ProcessTape
	  SavedState=0;

	  // Process the tape from a state file
	  NxCore.ProcessTape(StateFileFilename, NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
                    
	  // output return value from processTape(), helps in troubleshooting potential issues
	  NxCore.ProcessReturnValue(returnValue);

      // Unload NxCore DLL
	  NxCore.UnloadNxCore();
    }
    else {
	  cout << "Failed to load NxCore Library: " << dlerror() << endl;
	  exit(EXIT_FAILURE);
	}

	cout << "NxCore C++ SampleApp10 Stop." << endl;
	exit(EXIT_SUCCESS);
}


// The NXCore Callback Function	
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{ 

  // If this flag is set, NxCore has saved the state. Note that from the time
  // the NxCORESTATUS_SAVING_STATE message was received, the function issued
  // one moreNxCALLBACKRETURN_CONTINUE, and the next time in we will exit, thus
  // allowing NxCore to properly save the state and to exit automatically.
  if (SavedState)
      return NxCALLBACKRETURN_STOP;


  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
		// NxCore Status Message
	case NxMSG_STATUS:     
	     OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);			 
		 break;

    // NxCore Trade Message
	case NxMSG_TRADE:
		 OnNxCoreTrade(pNxCoreSys, pNxCoreMsg);
		 break;
  }
  
  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;  
}


// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{

    // If a Minute has elapsed print the NxCore system time.
    // NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
    // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
	  cout.fill('0');
	  cout << "NxCore Time: " <<
		setw(2) << (int) pNxCoreSys->nxDate.Month << "/" <<
		setw(2) << (int) pNxCoreSys->nxDate.Day << "/" <<
		setw(4) << (int) pNxCoreSys->nxDate.Year << " " <<

		setw(2) << (int) pNxCoreSys->nxTime.Hour << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Minute << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Second << endl;


	  // LOOK! If it's 7:00:00 AM SAVE THE CURRENT STATE TO A STATE FILE!
	  if (pNxCoreSys->nxTime.Hour==7 && pNxCoreSys->nxTime.Minute==0 && pNxCoreSys->nxTime.Second==0)
	  {
		// NOTE: This application will only work properly with the NxSAVESTATE_ONEPASS flag setting,
		// as we exit the tape exactly one pass after the file is to be saved. Should you modify this
		// flag the application will need to also be modified accordingly.
		NxCore.SaveState(StateFileFilename,NxSAVESTATE_ONEPASS);
	  }
	}
	
	// Print the specific NxCore status message
    switch( pNxCoreSys->Status ) 
    {
	  // NxCore will save the state this pass. Tell user and wait for keypress
  	  case NxCORESTATUS_SAVING_STATE:
		   cout << "NxCore Will Save the State Now!" <<  endl <<
		   	   "Press <ENTER> to Continue." << endl;
		   cin.get();

		   // Set the state saved flag
		   SavedState=1;
		   break;

	  // NxCore Has been loaded from a state tape. Tell user and wait for keypress
	  case NxCORESTATUS_LOADED_STATE:
		   cout << "NxCore Has Been Loaded From A State!" <<  endl <<
		   	   "Press <ENTER> to continue processing trades." << endl;
		   cin.get();
		   break;

	  case NxCORESTATUS_COMPLETE:
		   cout << "NxCore Complete Message." << endl;
		   break;

      case NxCORESTATUS_INITIALIZING:
		   cout << "NxCore Initialize Message." << endl;
		   break;

	  case NxCORESTATUS_SYNCHRONIZING:
		   cout << "NxCore Synchronizing Message." << endl;
		   break;

	  case NxCORESTATUS_WAITFORCOREACCESS:
		   cout << "NxCore Wait For Access." << endl;
		   break;

	  case NxCORESTATUS_RESTARTING_TAPE:
		   cout << "NxCore Restart Tape Message." << endl;
		   break;

      case NxCORESTATUS_ERROR:
		   cout << "NxCore Error." << endl;
		   break;
 	  
      case NxCORESTATUS_RUNNING:		   
		   break;

    }
}


// OnNxCoreTrade: Function to handle NxCore Trade messages.	
//--------------------------------------------------------------
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  if (pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='e')
  {
    // Assign a pointer to the Trade data
    NxCoreTrade* Trade = ( NxCoreTrade*) &pNxCoreMsg->coreData.Trade;

    // Get the price and net change
    double Price = NxCore.PriceToDouble(Trade->Price,Trade->PriceType);
    double NetChange = NxCore.PriceToDouble(Trade->NetChange, Trade->PriceType);

	// Write out Symbol, Time, Price, NetChg, Size, Reporting Exg
	cout.fill('0');
	cout << "Trade for Symbol: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String << "," <<
		" Time: " <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Hour << ":" <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Minute << ":" <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Second <<
		" Price: " 	<< fixed << setprecision(2) << Price <<
		" NetChg: " << fixed << setprecision(2) << NetChange <<
		" Size: " 	<< Trade->Size <<
		" Exchg: " 	<< pNxCoreMsg->coreHeader.ReportingExg << endl;
  }
}
