////////////////////////////////////////////////////
//// SampleApp11.cpp                            ////
//// Sample App for NxCore in C++               ////
//// Author: Jeffrey Donovan, Jeremy Truelove   ////
//// Date: 2017-02-0                            ////
////////////////////////////////////////////////////
//// Demonstrates:                              ////
//// Handling NxCore Trade and Quote Messages   ////
////////////////////////////////////////////////////
//// To Read as Written:                        ////
//// Tab Size: 4  Indent Size: 2, Keep Tabs     ////
////////////////////////////////////////////////////

#include "StdAfx.h"

#include <cstdlib>
#include <iostream>    				// cout, endl
#include <iomanip>					// setw, fixed, setprecisioncd ..

#include <cstdint>					// int64_t
#include "NxCoreAPI_Wrapper_C++.h"  // NxCore C++ wrapper with NxCoreClass

using namespace std;

// JTime Time Structure
typedef struct
{
  unsigned char Hour;
  unsigned char Minute;
  unsigned char Second;
  unsigned short Millisecond;
  unsigned short Microsecond;
  unsigned int MsOfDay;
  int64_t MicrosOfDay;
}JTime;

// Couple forwards - functions are below
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg);
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreExgQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);
void OnNxCoreMMQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);

// Global Instance of NxCore Class
NxCoreClass NxCore;

// Main Entry Point for app.	
int main(int argc, char* argv[])
{
	int returnValue;

	if(argc < 2) {
		cout << "SampleApp11 dll-file [tape-name]" << endl;
		exit(EXIT_SUCCESS);
	}
    cout << "NxCore C++ SampleApp11 Start." << endl;

	if (NxCore.LoadNxCore(argv[1]))
    {        
	  // If a tape filename was passed in command line argument,
	  // call ProcessTape with that argument. 
	  if (argv[2]){
		cout << "Processing the tape: " << argv[2] << endl;
	    returnValue = NxCore.ProcessTape(argv[2], NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
	  }
	  else
		returnValue = NxCore.ProcessTape("", NULL, 0, 0, (NxCoreCallback)OnNxCoreCallback);
                    
	  // output return value from processTape(), helps in troubleshooting potential issues
	  NxCore.ProcessReturnValue(returnValue);

      // Unload NxCore DLL
	  NxCore.UnloadNxCore();
    }
    else {
	  cout << "Failed to load NxCore Library: " << dlerror() << endl;
	  exit(EXIT_FAILURE);
	}

	cout << "NxCore C++ SampleApp11 Stop." << endl;
	exit(EXIT_SUCCESS);
}


// The NXCore Callback Function	
//-----------------------------
int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{ 
  // Do something based on the message type
  switch (pNxCoreMsg->MessageType)
  {
	// NxCore Status Message
	case NxMSG_STATUS:
	     OnNxCoreStatus(pNxCoreSys, pNxCoreMsg);
		 break;

    // NxCore Trade Message
	case NxMSG_TRADE:
		 OnNxCoreTrade(pNxCoreSys, pNxCoreMsg);
		 break;

	// NxCore Level1 Quote Message
	case NxMSG_EXGQUOTE:
		 OnNxCoreExgQuote(pNxCoreSys, pNxCoreMsg);
		 break;

	// NxCore Level2 Quote Message
	case NxMSG_MMQUOTE:
		 OnNxCoreMMQuote(pNxCoreSys, pNxCoreMsg);
		 break;
  }
	  
  // Continue running the tape
  return NxCALLBACKRETURN_CONTINUE;  
}


// OnNxCoreStatus: Function to handle NxCore Status messages
//----------------------------------------------------------
void OnNxCoreStatus(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg)
{
	
	// If a Minute has elapsed print the NxCore system time.
	// NOTE: The last time sent is 24:00:00. 24:00:00 is not actually
    // a real time value
	if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE) &&
	    (pNxCoreSys->nxTime.Hour < 24))
	{
	  cout.fill('0');
	  cout << "NxCore Time: " <<
		setw(2) << (int) pNxCoreSys->nxDate.Month << "/" <<
		setw(2) << (int) pNxCoreSys->nxDate.Day << "/" <<
		setw(4) << (int) pNxCoreSys->nxDate.Year << " " <<

		setw(2) << (int) pNxCoreSys->nxTime.Hour << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Minute << ":" <<
		setw(2) << (int) pNxCoreSys->nxTime.Second << endl;
	}
	
	// Print the specific NxCore status message
    switch( pNxCoreSys->Status ) 
    {
	  case NxCORESTATUS_COMPLETE:
		   cout << "NxCore Complete Message." << endl;
		   break;

      case NxCORESTATUS_INITIALIZING:
		   cout << "NxCore Initialize Message." << endl;
		   cout << "libnx.so version is v" <<
				NxCore.GetMajorVersion(pNxCoreSys->DLLVersion) << "." <<
				NxCore.GetMinorVersion(pNxCoreSys->DLLVersion) << "." <<
				NxCore.GetBuildVersion(pNxCoreSys->DLLVersion) << endl;
		   break;

	  case NxCORESTATUS_SYNCHRONIZING:
		   cout << "NxCore Synchronizing Message." << endl;
		   break;

	  case NxCORESTATUS_WAITFORCOREACCESS:
		   cout << "NxCore Wait For Access." << endl;
		   break;

	  case NxCORESTATUS_RESTARTING_TAPE:
		   cout << "NxCore Restart Tape Message." << endl;
		   break;

      case NxCORESTATUS_ERROR:
		   cout << "NxCore Error." << endl;
		   break;
 	  
      case NxCORESTATUS_RUNNING:		   
		   break;

    }
}

// Get Micro Diff from MsOfDay
#define JTime_MicroDiffFromMSofDay(micros,ms) micros-((int64_t)ms*1000)

// Convert MicrosecondOfDay to JTime
void inline JTime_MicrosOfDayToTime(int64_t MicrosOfDay,JTime *Time)
{
   Time->MicrosOfDay=MicrosOfDay;
   
   Time->MsOfDay=(unsigned int)(MicrosOfDay/1000);   
   unsigned int Seconds=Time->MsOfDay/1000;
     
   Time->Hour       =(unsigned char) (Seconds/(60*60));
   Time->Minute     =(unsigned char) (Seconds/60%60);
   Time->Second     =(unsigned char) (Seconds%60);
   Time->Millisecond=(unsigned short)(Time->MsOfDay % 1000);
   Time->Microsecond=(unsigned short)(JTime_MicroDiffFromMSofDay(MicrosOfDay,Time->MsOfDay));         
}

// OnNxCoreTrade: Function to handle NxCore Trade messages.	
//--------------------------------------------------------------
void OnNxCoreTrade(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
  if (pNxCoreMsg->coreHeader.pnxStringSymbol->String[0]=='e')
  {
    // Assign a pointer to the Trade data
    NxCoreTrade* Trade = ( NxCoreTrade*) &pNxCoreMsg->coreData.Trade;

    // Get the price and net change
    double Price = NxCore.PriceToDouble(Trade->Price,Trade->PriceType);
    double NetChange = NxCore.PriceToDouble(Trade->NetChange, Trade->PriceType);

    // Write out Symbol, Time, Price, NetChg, Size, Reporting Exg, Trade Conditon
	cout << "Trade for Symbol: " << pNxCoreMsg->coreHeader.pnxStringSymbol->String << "," <<
		" Time: " <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Hour << ":" <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Minute << ":" <<
			setw(2) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Second << "." <<
			setw(3) << (int) pNxCoreMsg->coreHeader.nxExgTimestamp.Millisecond <<
		" Price: " 	<< fixed << setprecision(2) << Price <<
		" NetChg: " << fixed << setprecision(2) << NetChange <<
		" Size: " 	<< Trade->Size <<
		" Exchg:" 	<< pNxCoreMsg->coreHeader.ReportingExg <<
		" TC: (" 	<< (int) Trade->TradeCondition << ")";

    // add any extended trade condtions (.nx2 and .nx3 formats ONLY)
    if (Trade->ExtTradeConditions[0]!=0) {
    	cout << " (" <<
			(int) Trade->ExtTradeConditions[0] << "," <<
			(int) Trade->ExtTradeConditions[1] << "," <<
			(int) Trade->ExtTradeConditions[2] << "," <<
			(int) Trade->ExtTradeConditions[3] << ")";
    }

    // always write out endln
    cout << endl;

    // add any microsecond timestamps (.nx3 format ONLY)
    if (pNxCoreMsg->pnxCore3Ext)
    {
      JTime SRCTime,PTP1Time,PTP2Time;
      JTime_MicrosOfDayToTime(pNxCoreMsg->pnxCore3Ext->MicrosOfDaySRC, &SRCTime);
      JTime_MicrosOfDayToTime(pNxCoreMsg->pnxCore3Ext->MicrosOfDayPTP1,&PTP1Time);
      JTime_MicrosOfDayToTime(pNxCoreMsg->pnxCore3Ext->MicrosOfDayPTP2,&PTP2Time);

	  cout.fill('0');
	  cout << "[SRC TIME: " <<
		setw(2) << (int) SRCTime.Hour << ":" <<
		setw(2) << (int) SRCTime.Minute << ":" <<
		setw(2) << (int) SRCTime.Second << "." <<
		setw(3) << (int) SRCTime.Millisecond << "." <<
		setw(3) << (int) SRCTime.Microsecond << "] "
		"  [PTP1 TIME: " <<
		setw(2) << (int) PTP1Time.Hour << ":" <<
		setw(2) << (int) PTP1Time.Minute << ":" <<
		setw(2) << (int) PTP1Time.Second << "." <<
		setw(3) << (int) PTP1Time.Millisecond << "." <<
		setw(3) << (int) PTP1Time.Microsecond << "] "
		"  [PTP2 TIME: " <<
		setw(2) << (int) PTP2Time.Hour << ":" <<
		setw(2) << (int) PTP2Time.Minute << ":" <<
		setw(2) << (int) PTP2Time.Second << "." <<
		setw(3) << (int) PTP2Time.Millisecond << "." <<
		setw(3) << (int) PTP2Time.Microsecond << "] " << endl;
    }
  }
}

	
// OnNxCoreQuote: Function to handle NxCore ExgQuote messages.	
//--------------------------------------------------------------
void OnNxCoreExgQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{

}

	
// OnNxCoreMMQuote: Function to handle NxCore MMQuote messages.	
//--------------------------------------------------------------
void OnNxCoreMMQuote(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg)
{
}
