/////////////////////////////////////////////////////
//// SampleApp12.cpp                             ////
//// Sample App for NxCore in C++                ////
//// Authors: Robert Garrett                     ////
//// Date: 2017-02-09                            ////
/////////////////////////////////////////////////////
//// Demonstrates:                               ////
//// Starting ProcessTape from multiple threads  ////
//// and save and reloading from multiple threads////
/////////////////////////////////////////////////////

#include "StdAfx.h"

#include <cstdlib>
#include <iostream>    				// cout, endl
#include <iomanip>					// setw, fixed, setprecision
#include "NxCoreAPI_Wrapper_C++.h"  // NxCore C++ wrapper with NxCoreClass
#include <thread>
#include <cstdio>

#define MAX_THREADS 10

NxCoreClass NxCore;
typedef struct {
	std::thread *myThread;
	int         threadNum;
	bool        tapeComplete;
	bool 	    savingState;
	char        stateFilename[256];
} ThreadData;

int  nextThread = 0;
char *tapeName;
thread_local ThreadData *myThreadData;

void ThreadProc(ThreadData *threadData);
int  OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg);

inline char *PrintTime(const NxCoreSystem* pNxCoreSys, char *timeBuf) {
	sprintf(timeBuf, "Thread %d: NxCoreTime: %02d/%02d/%02d %02d:%02d:%02d ", myThreadData->threadNum,
			pNxCoreSys->nxDate.Month, pNxCoreSys->nxDate.Day, pNxCoreSys->nxDate.Year % 100,
			pNxCoreSys->nxTime.Hour, pNxCoreSys->nxTime.Minute, pNxCoreSys->nxTime.Second);
	return timeBuf;
}

int main(int argc, char* argv[])
{
	int numThreads;
	ThreadData thread[MAX_THREADS];

	if(argc < 3) {
		cout << "SampleApp11 dll-file numThreads [tape-name]" << endl;
		exit(EXIT_SUCCESS);
	}
	numThreads = atoi(argv[2]);
	if(numThreads < 0 || numThreads > MAX_THREADS) {
		cout << "Number of threads must be between 1 and 10" << endl;
		exit(EXIT_SUCCESS);
	}
	tapeName = argv[3];

	cout << "NxCore C++ SampleApp12 Start." << endl;

	if (NxCore.LoadNxCore(argv[1]))
    {
		for(int i = 0; i < numThreads; ++i) {
			thread[i].threadNum = ++nextThread;
			thread[i].myThread = new std::thread(ThreadProc, &thread[i]);
		}

		// Wait for threads to exit.
		for(int i = 0; i < numThreads; ++i) {
			thread[i].myThread->join();
			delete thread[i].myThread;
		}

		// Unload NxCore DLL
		NxCore.UnloadNxCore();
    }
    else {
	  cout << "Failed to load NxCore Library: " << dlerror() << endl;
	  exit(EXIT_FAILURE);
	}

	cout << "NxCore C++ SampleApp11 Stop." << endl;
	exit(EXIT_SUCCESS);
}

void ThreadProc(ThreadData *threadData) {
	int returnValue;
	char *myTapeName = tapeName ? tapeName : nullptr;

	myThreadData = threadData;
	sprintf(threadData->stateFilename, "%s.%d", tapeName ? tapeName : "Current", threadData->threadNum);
	threadData->tapeComplete = false;
	threadData->savingState = false;


	while(!threadData->tapeComplete) {
		returnValue = NxCore.ProcessTape(myTapeName ? myTapeName : "", nullptr, 0, 0, (NxCoreCallback)OnNxCoreCallback);
		myTapeName = threadData->stateFilename;
		cout << "Thread " << threadData->threadNum << " completed: Status follows..." << endl;
		NxCore.ProcessReturnValue(returnValue);
	}
}

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys,const NxCoreMessage* pNxCoreMsg) {
	char timeBuf[64];

	// Only process status messages
	if(pNxCoreMsg->MessageType != NxMSG_STATUS) {
		return NxCALLBACKRETURN_CONTINUE;
	}

	// Print the specific NxCore status message
	switch (pNxCoreSys->Status) {
		case NxCORESTATUS_COMPLETE:
			myThreadData->tapeComplete = true;
			cout << "Thread " << myThreadData->threadNum << ": NxCore Complete Message." << endl;
			break;

		case NxCORESTATUS_INITIALIZING:
			cout << "Thread " << myThreadData->threadNum << ": NxCore Initialize Message. libnx.so version is v"
				 << NxCore.GetMajorVersion(pNxCoreSys->DLLVersion) << "."
				 << NxCore.GetMinorVersion(pNxCoreSys->DLLVersion) << "."
				 << NxCore.GetBuildVersion(pNxCoreSys->DLLVersion) << endl;
			break;

		case NxCORESTATUS_WAITFORCOREACCESS:
			cout << PrintTime(pNxCoreSys, timeBuf) << ": NxCore Wait For Access." << endl;
			break;

		case NxCORESTATUS_ERROR:
			cout << PrintTime(pNxCoreSys, timeBuf) << ": NxCore Error." << endl;
			break;

		case NxCORESTATUS_LOADED_STATE:
			cout << PrintTime(pNxCoreSys, timeBuf) << ": Loaded from state file " << myThreadData->stateFilename << endl;
			break;

		case NxCORESTATUS_SAVING_STATE:
			switch(pNxCoreSys->StatusData) {
			case NxCSSAVESTATE_CAPTURE:
				myThreadData->savingState = true;
				cout << PrintTime(pNxCoreSys, timeBuf)  << ": Saving state file " << myThreadData->stateFilename << endl;
				break;
			case NxCSSAVESTATE_COMPLETE:
				myThreadData->savingState = false;
				cout << PrintTime(pNxCoreSys, timeBuf) << ": State file " << myThreadData->stateFilename << " completed"<< endl;
				return NxCALLBACKRETURN_STOP;
				break;
			case NxCSSAVESTATE_ERR_CREATEFILE:
				myThreadData->savingState = false;
				cout << PrintTime(pNxCoreSys, timeBuf) << ": Save state error: CreateFile " << myThreadData->stateFilename << endl;
				break;
			case NxCSSAVESTATE_ERR_DISKSPACE:
				myThreadData->savingState = false;
				cout << PrintTime(pNxCoreSys, timeBuf) << ": Save state error: ErrDiskspace " << endl;
				break;
			case NxCSSAVESTATE_ERR_MEMORY:
				myThreadData->savingState = false;
				cout << PrintTime(pNxCoreSys, timeBuf) << ": Save state error: ErrMemory " << endl;
				break;
			}
			break;

		default:
			if ((pNxCoreSys->ClockUpdateInterval >= NxCLOCK_MINUTE)	&& (pNxCoreSys->nxTime.Hour < 24)) {
				cout << PrintTime(pNxCoreSys, timeBuf) << endl;
				if(!myThreadData->savingState && (pNxCoreSys->nxTime.Minute == 0 || pNxCoreSys->nxTime.Minute == 30)) {
					myThreadData->savingState = true;
					NxCore.SaveState(myThreadData->stateFilename, myThreadData->threadNum % 2 ? NxSAVESTATE_ONEPASS : NxSAVESTATE_GRADUALLY);
				}
			}
	}
    return NxCALLBACKRETURN_CONTINUE;
}

